@@ -325,6 +325,7 @@ namespace pan
 
       typedef std::map<Quark,ArticleNode*> nodes_t;
       typedef std::vector<ArticleNode*> nodes_v;
+      typedef std::vector<const ArticleNode*> const_nodes_v;
 
       struct GroupHeaders
       {
@@ -346,15 +347,21 @@ namespace pan
         ArticleNode* find_node (const Quark& mid);
         const ArticleNode* find_node (const Quark& mid) const;
 
-        Quark find_parent_message_id (const Quark& mid) const;
-        const Article* find_top_existing_ancestor (const Quark& mid) const;
+        const Quark& find_parent_message_id (const Quark& mid) const;
         Article* find_article (const Quark& mid);
         const Article* find_article (const Quark& mid) const;
-        void find_nodes (const quarks_t& mids, nodes_v& setme);
         void remove_articles (const quarks_t& mids);
         void build_references_header (const Article* article, std::string& setme) const;
       };
 
+      static void find_nodes (const quarks_t           & mids,
+                              nodes_t                  & nodes,
+                              nodes_v                  & setme);
+
+      static void find_nodes (const quarks_t           & mids,
+                              const nodes_t            & nodes,
+                              const_nodes_v            & setme);
+
       virtual void get_article_references (const Quark& group, const Article*, std::string& setme) const;
 
       /**
@@ -402,11 +409,9 @@ namespace pan
                                    const FilterInfo  * criteria  = 0);
 
         public:
-          void add_articles (const quarks_t&, bool apply_filter=true);
-          void remove_articles (const quarks_t& mids);
-
-        private:
-          void accumulate_descendants (quarks_t&, const ArticleNode*) const;
+          void articles_changed (const quarks_t& mids, bool do_refilter);
+          void add_articles     (const quarks_t& mids);
+          void remove_articles  (const quarks_t& mids);
 
         private: // implementation fields
           const Quark _group;
@@ -414,13 +419,21 @@ namespace pan
           nodes_t _nodes;
           std::deque<ArticleNode> _node_chunk;
           FilterInfo _filter;
+          Data::ShowType _show_type;
           struct NodeMidCompare;
+          struct TwoNodes;
+
+        private:
+          typedef std::set<const ArticleNode*,NodeMidCompare> unique_nodes_t;
+          void accumulate_descendants (unique_nodes_t&, const ArticleNode*) const;
+          void add_articles (const const_nodes_v&);
+          void apply_filter (const const_nodes_v&);
       };
 
       std::set<MyTree*> _trees;
       void on_articles_removed (const quarks_t& mids) const;
       void on_articles_added (const Quark& group, const quarks_t& mids);
-      void on_articles_changed (const quarks_t& mids) const;
+      void on_articles_changed (const quarks_t& mids, bool do_refilter) const;
       void remove_articles_from_tree (MyTree*, const quarks_t& mids) const;
       void add_articles_to_tree (MyTree*, const quarks_t& mids);
 
@@ -455,16 +468,16 @@ namespace pan
                               bool                         all_items_must_be_true,
                               const Scorefile::AddItem   * items,
                               size_t                       item_count,
-                              bool                         rescore_our_trees);
+                              bool                         do_rescore);
 
       virtual void comment_out_scorefile_line (const StringView    & filename,
                                                size_t                begin_line,
                                                size_t                end_line,
                                                bool                  do_rescore);
 
-    private:
+      virtual void rescore ();
 
-      void rescore ();
+    private:
 
       Scorefile _scorefile;
 
