@@ -26,6 +26,7 @@
 #include <cmath>
 extern "C" {
   #include <glib/gi18n.h>
+  #include <glib/gmessages.h> // for g_assert
 }
 #include <pan/general/debug.h>
 #include <pan/general/foreach.h>
@@ -70,29 +71,15 @@ DataImpl :: GroupHeaders :: find_node (const Quark& mid) const
   return node;
 }
 
-Quark
-DataImpl :: GroupHeaders :: find_parent_message_id (const Quark& message_id) const
+const Quark&
+DataImpl :: GroupHeaders :: find_parent_message_id (const Quark& mid) const
 {
-  Quark parent_mid;
-  const ArticleNode * node (find_node (message_id));
+  const ArticleNode * node (find_node (mid));
   if (node && node->_parent)
-    parent_mid = node->_parent->_mid;
-  return parent_mid;
-}
+    return node->_parent->_mid;
 
-const Article*
-DataImpl :: GroupHeaders :: find_top_existing_ancestor (const Quark& message_id) const
-{
-  const Article *a(0);
-
-  const ArticleNode * node (find_node (message_id));
-  while (node) {
-    if (node->_article)
-      a = node->_article;
-    node = node->_parent;
-  }
-
-  return a;
+  static const Quark empty_quark;
+  return empty_quark;
 }
 
 const Article*
@@ -119,37 +106,14 @@ DataImpl :: GroupHeaders :: find_article (const Quark& message_id)
   return a;
 }
 
-void
-DataImpl :: GroupHeaders :: find_nodes (const quarks_t& mids, nodes_v& setme)
-{
-  setme.reserve (mids.size());
-  nodes_t::iterator node_it  (_nodes.begin()), node_end (_nodes.end());
-  quarks_t::const_iterator mid_it (mids.begin()), mid_end (mids.end());
-
-  while (node_it!=node_end && mid_it!=mid_end)
-  {
-    if (node_it->first < *mid_it)
-      ++node_it;
-    else if (*mid_it < node_it->first) // looks like we won't find *mid_it
-      ++mid_it;
-    else { // equal
-      setme.push_back (node_it->second);
-      ++node_it;
-      ++mid_it;
-    }
-  }
-}
-
 void
 DataImpl :: GroupHeaders :: remove_articles (const quarks_t& mids)
 {
-  const size_t old_size (_nodes.size());
-  foreach_const (quarks_t, mids, it)
-    _nodes.erase (*it);
-  if (old_size != _nodes.size()) {
-    //std::cerr << LINE_ID << " setting dirty flag to true..." << std::endl;
-    _dirty = true;
-  }
+  nodes_v nodes;
+  find_nodes (mids, _nodes, nodes);
+  foreach (nodes_v, nodes, it)
+    (*it)->_article = 0;
+  _dirty = true;
 }
   
 const DataImpl :: GroupHeaders*
@@ -233,6 +197,54 @@ DataImpl :: unref_group   (const Quark& group)
   }
 }
 
+void
+DataImpl :: find_nodes (const quarks_t           & mids,
+                        nodes_t                  & nodes,
+                        nodes_v                  & setme)
+{
+  nodes_t::iterator nit(nodes.begin()), nend(nodes.end());
+  quarks_t::const_iterator mit(mids.begin()), mend(mids.end());
+
+  while (mit!=mend && nit!=nend) {
+    const Quark& m_mid (*mit);
+    const Quark& n_mid (nit->second->_mid);
+    if (m_mid < n_mid)
+      mit = std::lower_bound (mit, mend, n_mid);
+    else if (n_mid < m_mid)
+      nit = nodes.lower_bound (m_mid);
+    else {
+      g_assert (m_mid == n_mid);
+      setme.push_back (nit->second);
+      ++mit;
+      ++nit;
+    }
+  }
+}
+
+void
+DataImpl :: find_nodes (const quarks_t           & mids,
+                        const nodes_t            & nodes,
+                        const_nodes_v            & setme)
+{
+  nodes_t::const_iterator nit(nodes.begin()), nend(nodes.end());
+  quarks_t::const_iterator mit(mids.begin()), mend(mids.end());
+
+  while (mit!=mend && nit!=nend) {
+    const Quark& m_mid (*mit);
+    const Quark& n_mid (nit->second->_mid);
+    if (m_mid < n_mid)
+      mit = std::lower_bound (mit, mend, n_mid);
+    else if (n_mid < m_mid)
+      nit = nodes.lower_bound (m_mid);
+    else {
+      g_assert (m_mid == n_mid);
+      setme.push_back (nit->second);
+      ++mit;
+      ++nit;
+    }
+  }
+}
+
 /*******
 ********
 *******/
@@ -300,7 +312,7 @@ DataImpl :: load_article (const Quark       & group,
         static const ArticleNode blank_node;
         h->_node_chunk.push_back (blank_node);
         new_parent_node = h->_nodes[new_parent_mid] = &h->_node_chunk.back();
-        new_parent_node->_mid = tok;
+        new_parent_node->_mid = new_parent_mid;
       }
       node->_parent = new_parent_node;
       if (find_ancestor (new_parent_node, new_parent_mid)) {
@@ -338,7 +350,7 @@ DataImpl :: load_article (const Quark       & group,
         static const ArticleNode blank_node;
         h->_node_chunk.push_back (blank_node);
         new_parent_node = h->_nodes[new_parent_mid] = &h->_node_chunk.back();
-        new_parent_node->_mid = tok;
+        new_parent_node->_mid = new_parent_mid;
       }
       node->_parent = new_parent_node;
       if (find_ancestor (new_parent_node, new_parent_mid) != 0) {
@@ -865,7 +877,7 @@ DataImpl :: mark_read (const Article  ** articles,
   }
 
   // fire an articles changed event...
-  on_articles_changed (mids);
+  on_articles_changed (mids, false);
 }
 
 
@@ -897,6 +909,7 @@ DataImpl :: get_article_scores (const Quark         & group,
 void
 DataImpl :: rescore ()
 {
+  //std::cerr << LINE_ID << " rescoring... " << std::endl;
   const std::string filename (_data_io->get_scorefile_name());
 
   // reload the scorefile...
@@ -913,6 +926,7 @@ DataImpl :: rescore ()
   foreach_const (quarks_t, groups, git)
   {
     const Quark& group (*git);
+    //std::cerr << LINE_ID << " rescoring " << group << std::endl;
 
     ArticleFilter::sections_t score_sections;
     _scorefile.get_matching_sections (StringView(group), score_sections);
@@ -922,16 +936,20 @@ DataImpl :: rescore ()
       Article * article (nit->second->_article);
       if (article) {
         const int old_score (article->score);
-        article->score = _article_filter.score_article (*this, score_sections, group, *article);
+        article->score = _article_filter.score_article (
+                                      *this, score_sections, group, *article);
         if (old_score != article->score)
           changed_mids.insert (article->message_id);
       }
     }
   }
 
+  //std::cerr << LINE_ID << ' ' << changed_mids.size()
+  //                     << " articles changed score\n";
+
   // event notification...
   if (!changed_mids.empty())
-    on_articles_changed (changed_mids);
+    on_articles_changed (changed_mids, true);
 }
 
 void
@@ -942,7 +960,7 @@ DataImpl :: add_score (const StringView           & section_wildmat,
                        bool                         all_items_must_be_true,
                        const Scorefile::AddItem   * items,
                        size_t                       item_count,
-                       bool                         rescore_trees)
+                       bool                         do_rescore)
 {
   const std::string filename (_data_io->get_scorefile_name());
 
@@ -957,7 +975,7 @@ DataImpl :: add_score (const StringView           & section_wildmat,
     o.close ();
   }
 
-  if (rescore_trees)
+  if (do_rescore)
     rescore ();
 }
 
@@ -1021,6 +1039,11 @@ DataImpl :: group_clear_articles (const Quark& group)
     delete_articles (all);
   }
 
+  // reset GroupHeaders' memory...
+//  headers->_nodes.clear ();
+//  headers->_node_chunk.clear ();
+//  headers->_art_chunk.clear ();
+
   // remove 'em from disk too.
   _data_io->clear_group_headers (group);
 
@@ -1083,23 +1106,10 @@ DataImpl :: on_articles_removed (const quarks_t& mids) const
 }
 
 void
-DataImpl :: on_articles_changed (const quarks_t& mids) const
+DataImpl :: on_articles_changed (const quarks_t& mids, bool do_refilter) const
 {
-  foreach (std::set<MyTree*>, _trees, tit)
-  {
-    // build a set of changed mids that are in this tree
-    quarks_t mids_in_tree;
-    foreach (quarks_t, mids, mit)
-      if ((*tit)->_nodes.count (*mit))
-        mids_in_tree.insert (*mit);
-
-    // fire an event if any of the mids _were_ in this tree
-    if (!mids_in_tree.empty()) {
-      ArticleTree::Diffs diffs;
-      diffs.changed = mids_in_tree;
-      (*tit)->fire_diffs (diffs);
-    }
-  }
+  foreach (std::set<MyTree*>, _trees, it)
+    (*it)->articles_changed (mids, do_refilter);
 }
 
 void
