@@ -190,7 +190,7 @@ DataImpl :: xover_unref (const Quark& group)
     on_articles_added (group, workarea._added_batch);
     workarea._added_batch.clear();
 
-    on_articles_changed (workarea._changed_batch);
+    on_articles_changed (workarea._changed_batch, true);
     workarea._changed_batch.clear();
 
     xover_clear_workarea (group);
@@ -310,7 +310,7 @@ DataImpl :: xover_add (const Quark         & server,
   {
     on_articles_added (group, workarea._added_batch);
     workarea._added_batch.clear();
-    on_articles_changed (workarea._changed_batch);
+    on_articles_changed (workarea._changed_batch, true);
     workarea._changed_batch.clear();
     workarea._batch_parts_size = 0;
 
