@@ -487,12 +487,14 @@ namespace pan
                                bool                         all_items_must_be_true,
                                const Scorefile::AddItem   * items,
                                size_t                       item_count,
-                               bool                         rescore_our_trees) = 0;
+                               bool                         do_rescore) = 0;
 
        virtual void comment_out_scorefile_line (const StringView    & filename,
                                                 size_t                begin_line,
                                                 size_t                end_line,
-                                                bool                  rescore_our_trees) = 0;
+                                                bool                  do_rescore) = 0;
+
+        virtual void rescore () = 0;
 
 
 
