@@ -55,6 +55,7 @@ namespace
     { icon_blue_flag, "ICON_BLUE_FLAG" }, 
     { icon_compose_followup, "ICON_COMPOSE_FOLLOWUP" }, 
     { icon_compose_post, "ICON_COMPOSE_POST" }, 
+    { icon_disk, "ICON_DISK" }, 
     { icon_filter_only_attachments, "ICON_ONLY_ATTACHMENTS" }, 
     { icon_filter_only_cached, "ICON_ONLY_CACHED" }, 
     { icon_filter_only_me, "ICON_ONLY_ME" }, 
@@ -276,17 +277,17 @@ namespace
     { "delete-groups-articles", GTK_STOCK_DELETE, N_("_Delete Group's Articles"), "<control><shift>Delete", N_("Delete Selected Groups' Articles"), G_CALLBACK(do_clear_selected_groups) },
     { "get-new-headers-in-selected-groups", "ICON_GET_SELECTED", N_("Get New _Headers in Selected Groups"), "A", N_("Get New Headers for Selected Groups"), G_CALLBACK(do_xover_selected_groups) },
     { "get-new-headers-in-subscribed-groups", "ICON_GET_SUBSCRIBED", N_("Get New _Headers in Subscribed Groups"), "<shift>A", N_("Get New Headers for Subscribed Groups"), G_CALLBACK(do_xover_subscribed_groups) },
-    { "download-headers", "ICON_GET_DIALOG", N_("Get _Headers..."), "", N_("Get Headers from Server"), G_CALLBACK(do_download_headers) },
+    { "download-headers", "ICON_GET_DIALOG", N_("Get _Headers ..."), "", N_("Get Headers from Server"), G_CALLBACK(do_download_headers) },
     { "refresh-group-list", NULL, N_("Refresh Group List"), "", N_("Get a new Group list from the Server"), G_CALLBACK(do_refresh_groups) },
     { "subscribe", NULL, N_("_Subscribe"), NULL, N_("Subscribe to Selected Groups"), G_CALLBACK(do_subscribe_selected_groups) },
     { "unsubscribe", NULL, N_("_Unsubscribe"), NULL, N_("Unsubscribe from Selected Groups"), G_CALLBACK(do_unsubscribe_selected_groups) },
 
-    { "save-articles", GTK_STOCK_SAVE, N_("_Save Articles..."), "<shift>S", N_("Save Selected Articles..."), G_CALLBACK(do_save_articles) },
+    { "save-articles", GTK_STOCK_SAVE, N_("_Save Articles ..."), "<shift>S", N_("Save Selected Articles ..."), G_CALLBACK(do_save_articles) },
     { "print", GTK_STOCK_PRINT, NULL, "<control>P", NULL, G_CALLBACK(do_print) },
-    { "import-tasks", GTK_STOCK_OPEN, N_("_Import NZB Files..."), NULL, NULL, G_CALLBACK(do_import_tasks) },
+    { "import-tasks", GTK_STOCK_OPEN, N_("_Import NZB Files ..."), NULL, NULL, G_CALLBACK(do_import_tasks) },
     { "cancel-last-task", GTK_STOCK_CANCEL, N_("_Cancel Last Task"), "<control>Delete", N_("Cancel Last Task"), G_CALLBACK(do_cancel_latest_task) },
-    { "show-task-window", NULL, N_("_Task Manager..."), NULL, NULL, G_CALLBACK(do_show_task_window) },
-    { "show-log-dialog", GTK_STOCK_DIALOG_INFO, N_("_Event Log..."), NULL, NULL, G_CALLBACK(do_show_log_window) },
+    { "show-task-window", NULL, N_("_Task Manager"), NULL, NULL, G_CALLBACK(do_show_task_window) },
+    { "show-log-dialog", GTK_STOCK_DIALOG_INFO, N_("_Event Log"), NULL, NULL, G_CALLBACK(do_show_log_window) },
     { "quit", NULL, N_("_Quit"), "<control>Q", NULL, G_CALLBACK(do_quit) },
 
     { "select-all-articles", NULL, N_("Select _All Articles"), "<control>D", NULL, G_CALLBACK(do_select_all_articles) },
@@ -295,10 +296,10 @@ namespace
     { "add-threads-to-selection", NULL, N_("Add _Threads to Selection"), "<shift>D", NULL, G_CALLBACK(do_add_threads_to_selection) },
     { "add-similar-articles-to-selection", NULL, N_("Add _Similar Articles to Selection"), "<control>S", NULL, G_CALLBACK(do_add_similar_to_selection) },
     { "select-article-body", NULL, N_("Select Article _Body"), NULL, NULL, G_CALLBACK(do_select_article_body) },
-    { "show-preferences-dialog", GTK_STOCK_PREFERENCES, N_("Edit _Preferences..."), NULL, NULL, G_CALLBACK(do_show_preferences_dialog) },
-    { "show-group-preferences-dialog", GTK_STOCK_PREFERENCES, N_("Edit _Group Preferences..."), NULL, NULL, G_CALLBACK(do_show_group_preferences_dialog) },
-    { "show-profiles-dialog", GTK_STOCK_EDIT, N_("Edit P_osting Profiles..."), NULL, NULL, G_CALLBACK(do_show_profiles_dialog) },
-    { "show-servers-dialog", GTK_STOCK_NETWORK, N_("Edit _News Servers..."), NULL, NULL, G_CALLBACK(do_show_servers_dialog) },
+    { "show-preferences-dialog", GTK_STOCK_PREFERENCES, N_("Edit _Preferences"), NULL, NULL, G_CALLBACK(do_show_preferences_dialog) },
+    { "show-group-preferences-dialog", GTK_STOCK_PREFERENCES, N_("Edit _Group Preferences"), NULL, NULL, G_CALLBACK(do_show_group_preferences_dialog) },
+    { "show-profiles-dialog", GTK_STOCK_EDIT, N_("Edit P_osting Profiles"), NULL, NULL, G_CALLBACK(do_show_profiles_dialog) },
+    { "show-servers-dialog", GTK_STOCK_NETWORK, N_("Edit _News Servers"), NULL, NULL, G_CALLBACK(do_show_servers_dialog) },
 
     { "jump-to-group-tab", GTK_STOCK_JUMP_TO, N_("Jump to _Group Tab"), "1", NULL, G_CALLBACK(do_jump_to_group_tab) },
     { "jump-to-header-tab", GTK_STOCK_JUMP_TO, N_("Jump to _Header Tab"), "2", NULL, G_CALLBACK(do_jump_to_header_tab) },
@@ -308,8 +309,8 @@ namespace
     { "clear-header-pane", NULL, N_("Clear _Header Pane"), NULL, N_("Clear Header Pane"), G_CALLBACK(do_clear_header_pane) },
     { "clear-body-pane", NULL, N_("Clear _Body Pane"), NULL, N_("Clear Body Pane"), G_CALLBACK(do_clear_body_pane) },
 
-    { "download-selected-article", NULL, N_("Download _Selected Article"), NULL, N_("Download Selected Article"), G_CALLBACK(do_download_selected_article) },
-    { "read-selected-article", "ICON_READ_MORE", N_("Read _Selected Article"), NULL, N_("Read Selected Article"), G_CALLBACK(do_read_selected_article) },
+    { "download-selected-article", "ICON_DISK", N_("Cache Article"), NULL, N_("Cache Article"), G_CALLBACK(do_download_selected_article) },
+    { "read-selected-article", "ICON_READ_MORE", N_("Read Article"), NULL, N_("Read Article"), G_CALLBACK(do_read_selected_article) },
     { "read-more", "ICON_READ_MORE", N_("Read _More"), "space", N_("Read More"), G_CALLBACK(do_read_more) },
     { "read-less", "ICON_READ_LESS", N_("Read _Back"), "BackSpace", N_("Read Back"), G_CALLBACK(do_read_less) },
     { "read-next-unread-group", "ICON_READ_UNREAD_GROUP", N_("Next _Unread Group"), "G", N_("Next Unread Group"), G_CALLBACK(do_read_next_unread_group) },
@@ -323,28 +324,28 @@ namespace
     { "read-previous-thread", NULL, N_("Previous _Thread"), "<control>V", N_("Previous Thread"), G_CALLBACK(do_read_previous_thread) },
     { "read-parent-article", NULL, N_("_Parent Article"), "U", N_("Parent Article"), G_CALLBACK(do_read_parent_article) },
 
-    { "plonk", NULL, N_("_Plonk Author"), NULL, NULL, G_CALLBACK(do_plonk) },
+    { "plonk", NULL, N_("Ignore _Author"), NULL, NULL, G_CALLBACK(do_plonk) },
     { "watch-thread", NULL, N_("_Watch Thread"), NULL, NULL, G_CALLBACK(do_watch) },
     { "ignore-thread", NULL, N_("_Ignore Thread"), NULL, NULL, G_CALLBACK(do_ignore) },
-    { "view-article-score", "ICON_SCORE", N_("View Article's S_core"), "<control><shift>C", NULL, G_CALLBACK(do_show_score_dialog) },
-    { "add-article-score", "ICON_SCORE", N_("_Add a _Scoring Rule"), "S", NULL, G_CALLBACK(do_show_new_score_dialog) },
-    { "cancel-article", GTK_STOCK_CANCEL, N_("Cance_l Article..."), NULL, NULL, G_CALLBACK(do_cancel_article) },
-    { "supersede-article", GTK_STOCK_CONVERT, N_("_Supersede Article..."), NULL, NULL, G_CALLBACK(do_supersede_article) },
+    { "view-article-score", "ICON_SCORE", N_("Edit Article's Watch/Ignore/Score ..."), "<control><shift>C", NULL, G_CALLBACK(do_show_score_dialog) },
+    { "add-article-score", "ICON_SCORE", N_("_Add a _Scoring Rule ..."), "S", NULL, G_CALLBACK(do_show_new_score_dialog) },
+    { "cancel-article", GTK_STOCK_CANCEL, N_("Cance_l Article ..."), NULL, NULL, G_CALLBACK(do_cancel_article) },
+    { "supersede-article", GTK_STOCK_CONVERT, N_("_Supersede Article ..."), NULL, NULL, G_CALLBACK(do_supersede_article) },
     { "delete-article", GTK_STOCK_DELETE, N_("_Delete Article"), "Delete", NULL, G_CALLBACK(do_delete_article) },
     { "mark-article-read", "ICON_ARTICLE_READ", N_("_Mark Article as Read"), "M", N_("Mark Article as Read"), G_CALLBACK(do_mark_article_read) },
     { "mark-article-unread", "ICON_ARTICLE_UNREAD", N_("Mark Article as _Unread"), "<control>M", N_("Mark Article as Unread"), G_CALLBACK(do_mark_article_unread) },
 
     { "post", "ICON_COMPOSE_POST", N_("_Post to Newsgroup"), "P", N_("Post to Newsgroup"), G_CALLBACK(do_post) },
     { "followup-to", "ICON_COMPOSE_FOLLOWUP", N_("_Followup to  Newsgroup"), "F", N_("Followup to Newsgroup"), G_CALLBACK(do_followup_to) },
-    { "reply-to", "ICON_COMPOSE_FOLLOWUP", N_("_Reply to Author in Mail"), "R", N_("Reply to Author in Mail"), G_CALLBACK(do_reply_to) },
+    { "reply-to", NULL, N_("_Reply to Author in Mail"), "R", N_("Reply to Author in Mail"), G_CALLBACK(do_reply_to) },
 
-    { "pan-web-page", NULL, N_("_Pan Home Page..."), NULL, NULL, G_CALLBACK(do_pan_web) },
-    { "bug-report", NULL, N_("Give _Feedback or Report a Bug..."), NULL, NULL, G_CALLBACK(do_bug_report) },
-    { "tip-jar", NULL, N_("_Tip Jar..."), NULL, NULL, G_CALLBACK(do_tip_jar) },
+    { "pan-web-page", NULL, N_("_Pan Home Page"), NULL, NULL, G_CALLBACK(do_pan_web) },
+    { "bug-report", NULL, N_("Give _Feedback or Report a Bug ..."), NULL, NULL, G_CALLBACK(do_bug_report) },
+    { "tip-jar", NULL, N_("_Tip Jar ..."), NULL, NULL, G_CALLBACK(do_tip_jar) },
 #if GTK_CHECK_VERSION(2,6,0)
-    { "about-pan", GTK_STOCK_ABOUT, N_("_About..."), NULL, NULL, G_CALLBACK(do_about_pan) }
+    { "about-pan", GTK_STOCK_ABOUT, N_("_About"), NULL, NULL, G_CALLBACK(do_about_pan) }
 #else
-    { "about-pan", NULL, N_("_About..."), NULL, NULL, G_CALLBACK(do_about_pan) }
+    { "about-pan", NULL, N_("_About"), NULL, NULL, G_CALLBACK(do_about_pan) }
 #endif
   };
 
@@ -365,7 +366,6 @@ namespace
     { "show-smilies-as-graphics", NULL, N_("Show _Smilies as Graphics"),      NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
     { "show-text-markup",         NULL, N_("Show *Bold*, __Underlined__, and /Italic/"), NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
     { "size-pictures-to-fit",     NULL, N_("Size Pictures to _Fit"),          NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
-    { "smooth-scrolling",         NULL, N_("Smooth Scrolling"),               NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
     { "monospace-font-enabled",   NULL, N_("Use _Monospace Font"),             "C", NULL, G_CALLBACK(prefs_toggle_callback_impl), false },
 
     { "work-online", "ICON_ONLINE", N_("_Work Online"), "L", NULL, G_CALLBACK(do_work_online), true },
