@@ -365,7 +365,8 @@ namespace
         get_iter_from_event_coords (w, (int)event->x, (int)event->y, &iter);
         if (gtk_text_iter_has_tag (&iter, pix_tag))
         {
-          gtk_text_iter_backward_to_tag_toggle (&iter, pix_tag);
+          if (!gtk_text_iter_begins_tag (&iter, pix_tag))
+            gtk_text_iter_backward_to_tag_toggle (&iter, pix_tag);
           g_assert (gtk_text_iter_begins_tag (&iter, pix_tag));
           const int offset (gtk_text_iter_get_offset (&iter));
           GtkTextBuffer * buf (gtk_text_view_get_buffer (GTK_TEXT_VIEW(w)));
@@ -687,11 +688,13 @@ namespace
         const char * prev (0);
         for (;;) {
           const char * b (line.strpbrk ("_*/"));
-          if (!b) break; 
-          if (b != prev) {
+          if (!b) break;
+          // if it's a new character, and (the first char in the line OR previous char is space/punctuation)
+          if (b != prev && (b==&line.front() || (isspace(b[-1] || ispunct(b[-1]))))) {
             const StringView lineleft (line.substr (b+1, 0));
             const char * e = lineleft.strchr (*b);
-            if (e) {
+            // if there's a closing character and (it's the last character OR the following character is space/punctuation)
+            if (e && (e==&lineleft.back() || (isspace(e[1]) || ispunct(e[1])))) {
               const char * type (0);
               switch (*b) {
                 case '*': type = "bold"; break;
@@ -1398,14 +1401,14 @@ BodyPane :: create_followup_or_reply (bool is_reply)
 
     // Subject:
     StringView v = g_mime_message_get_subject (_message);
-    v = normalize_subject_re (v);
+    std::string h = header_to_utf8 (v, message_charset, group_charset);
+    v = normalize_subject_re (h);
     std::string val (v.str, v.len);
     if (!val.find("RE:") || !val.find ("Re:"))
       val.replace (0, 3, "Re:"); // be polite & force lowercase 'e'
     else
       val.insert (0, "Re: "); // no Re: -- add one.
-    std::string h = header_to_utf8 (val, message_charset, group_charset);
-    g_mime_message_set_subject (msg, h.c_str());
+    g_mime_message_set_subject (msg, val.c_str());
 
     // attribution lines
 
