@@ -40,23 +40,21 @@ namespace
 {
   enum ModelColumns
   {
-    COL_NAME,
     COL_TOTAL,
     COL_UNREAD,
     COL_QTY
   };
 
-  struct Row: public PanTreeStore::Row
+  struct MyRow: public PanTreeStore::Row
   {
     public:
-      Quark * name; // pointer to a Quark owned by groups_in_model
+      Quark groupname;
       unsigned long total;
       unsigned long unread;
 
     public:
       virtual void get_value (int column, GValue* setme) {
         switch (column) {
-          case COL_NAME:   set_value_pointer (setme, name);  break;
           case COL_TOTAL:  set_value_ulong   (setme, total);  break;
           case COL_UNREAD: set_value_ulong   (setme, unread); break;
         }
@@ -103,10 +101,10 @@ namespace
    * For that reason, callers should pass in a reference to a groups_in_model element:
    *   create_row (_tree_store, &iter, _data, *groups_in_model.insert(group).first);
    */
-  Row* create_row (const Data& data, Quark& groupname)
+  MyRow* create_row (const Data& data, const Quark& groupname)
   {
-    Row * row = new Row ();
-    row->name = &groupname;
+    MyRow * row = new MyRow ();
+    row->groupname = groupname;
     data.get_group_counts (groupname, row->unread, row->total);
     return row;
   }
@@ -128,45 +126,20 @@ namespace
       }
     }
   }
-
-
-  typedef std::vector<Quark> quark_v;
-
-  void
-  add_groups_to_model (const Data                & data,
-                       PanTreeStore              * store,
-                       GtkTreeIter               & parent_iter,
-                       quark_v                   & groups,
-                       size_t                      offset,
-                       size_t                      n_groups)
-  {
-    if (n_groups)
-    {
-      std::vector<PanTreeStore::Row*> rows;
-      for (size_t i(0); i!=n_groups; ++i)
-        rows.push_back (create_row (data, groups[offset+i]));
-      GtkTreeIter unused;
-      pan_tree_store_append_n (store, &unused, &parent_iter, rows);
-    }
-  }
 }
 
 Quark
 GroupPane :: get_selection () const
 {
-   Quark group_name;
+   Quark groupname;
 
    GtkTreeIter iter;
    GtkTreeModel * model;
    GtkTreeSelection * selection (gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view)));
-   if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
-      gpointer name (0);
-      gtk_tree_model_get (model, &iter, COL_NAME, &name, -1);
-      if (name)
-         group_name = *(Quark*)name;
-   }
+   if (gtk_tree_selection_get_selected (selection, &model, &iter))
+     groupname = dynamic_cast<MyRow*>(_tree_store->get_row (&iter))->groupname;
 
-   return group_name;
+   return groupname;
 }
 
 quarks_t
@@ -182,12 +155,8 @@ GroupPane :: get_full_selection () const
   for (GList *l(list); l; l=l->next) {
     GtkTreePath * path (static_cast<GtkTreePath*>(l->data));
     GtkTreeIter iter;
-    if (gtk_tree_model_get_iter (model, &iter, path)) {
-      gpointer name (0);
-      gtk_tree_model_get (model, &iter, COL_NAME, &name, -1);
-      if (name)
-        groups.insert (*(Quark*)name);
-    }
+    if (gtk_tree_model_get_iter (model, &iter, path))
+      groups.insert (dynamic_cast<MyRow*>(_tree_store->get_row (&iter))->groupname);
   }
 
   // cleanup
@@ -329,16 +298,17 @@ GroupPane :: on_popup_menu (GtkWidget *treeview, gpointer userdata)
 namespace
 {
   typedef std::vector<GtkTreeIter> tree_iters_t;
+  typedef std::pair<Quark,MyRow*> row_t;
+  typedef Loki::AssocVector<Quark,MyRow*> rows_t;
 
   PanTreeStore*
   build_model (const Data         & data,
                const TextMatch    * match,
-               tree_iters_t       & setme,
-               std::vector<Quark> & groups_in_model)
+               tree_iters_t       & expandme,
+               rows_t             & setme_rows)
   {
     // build the model...
     PanTreeStore * store = PanTreeStore :: new_tree (COL_QTY, 
-                                                     G_TYPE_POINTER, // name
                                                      G_TYPE_ULONG,   // total
                                                      G_TYPE_ULONG);  // unread
 
@@ -350,43 +320,48 @@ namespace
     data.get_subscribed_groups (groups);
     find_matching_groups (match, groups, sub);
 
-    // build the vector of quarks that we will keep.
-    // the pointer in COL_NAME points to a quark in this vector,
-    // so we populate it all at once to ensure the pointers don't
-    // change from a resize() later.
-    groups_in_model.clear ();
-    groups_in_model.reserve (unsub.size() + sub.size());
-    const size_t sub_low = groups_in_model.size();
-    const size_t sub_count = sub.size();
-    groups_in_model.insert (groups_in_model.end(), sub.begin(), sub.end());
-    const size_t unsub_low = groups_in_model.size();
-    const size_t unsub_count = unsub.size();
-    groups_in_model.insert (groups_in_model.end(), unsub.begin(), unsub.end());
+    std::vector<row_t> tmp_rows;
+    tmp_rows.reserve (sub.size() + unsub.size());
 
     // subscribed
-    Row * row = new Row ();
-    row->name = sub_title_quark;
-    GtkTreeIter iter;
-    pan_tree_store_append (store, &iter, NULL, row);
-    const GtkTreeIter sub_section = iter;
-    add_groups_to_model (data, store, iter, groups_in_model, sub_low, sub_count);
-    if (sub_count)
-      setme.push_back (sub_section);
+    MyRow * row = new MyRow ();
+    row->groupname = *sub_title_quark;
+    store->append (NULL, row);
+    if (!sub.empty()) {
+      std::vector<PanTreeStore::Row*> tmp (sub.size());
+      for (size_t i(0), n(sub.size()); i!=n; ++i) {
+        const Quark& groupname (sub[i]);
+        MyRow * r = create_row (data, groupname);
+        tmp[i] = r;
+        tmp_rows.push_back (row_t (groupname, r));
+      }
+      store->append (row, tmp);
+      expandme.push_back (store->get_iter (row));
+    }
 
     // unsubscribed
-    row = new Row ();
-    row->name = other_title_quark;
-    pan_tree_store_append (store, &iter, NULL, row);
-    const GtkTreeIter other_section = iter;
-    add_groups_to_model (data, store, iter, groups_in_model, unsub_low, unsub_count);
-    // only expand the unsub expander if user specified search criteria,
-    // or if there's no criteria and no subscribed groups (first-time usrs).
-    // otherwise it's a flood of thousands of groups.
-    // they can click the expander themselves to get that,
-    const bool user_did_search (match != 0);
-    if (unsub_count && (user_did_search || !unsub_count))
-      setme.push_back (other_section);
+    row = new MyRow ();
+    row->groupname = *other_title_quark;
+    store->append (NULL, row);
+    if (!unsub.empty()) {
+      std::vector<PanTreeStore::Row*> tmp (unsub.size());
+      for (size_t i(0), n(unsub.size()); i!=n; ++i) {
+        const Quark& groupname (unsub[i]);
+        MyRow * r = create_row (data, groupname);
+        tmp[i] = r;
+        tmp_rows.push_back (row_t (groupname, r));
+      }
+      store->append (row, tmp);
+      // only expand the unsub expander if user specified search criteria,
+      // or if there's no criteria and no subscribed groups (first-time usrs).
+      // otherwise it's a flood of thousands of groups.
+      // they can click the expander themselves to get that,
+      const bool user_did_search (match != 0);
+      if (!unsub.empty() && (user_did_search || sub.empty()))
+        expandme.push_back (store->get_iter (row));
+    }
 
+    setme_rows.assign (tmp_rows.begin(), tmp_rows.end());
     return store;
   }
 }
@@ -395,118 +370,70 @@ namespace
 ****  Data listener
 ***/
 
-namespace
-{
-  struct FindGroupWalker: public PanTreeStore::WalkFunctor
-  {
-    const Quark group;
-    GtkTreeIter found_it;
-    bool found;
-    FindGroupWalker (const Quark& q): group(q), found(false) {}
-    virtual ~FindGroupWalker () {}
-    virtual bool operator()(PanTreeStore *store, PanTreeStore::Row *r, GtkTreeIter *it, GtkTreePath *path) {
-      const Row * row (dynamic_cast<Row*>(r));
-      if (group == *row->name) {
-        found_it = *it;
-        found = true;
-      }
-      const bool more = !found;
-      return more;
-    }
-  };
-
-  bool
-  find_group (GtkTreeModel* model, GtkTreeIter& setme, const Quark& group)
-  {
-    FindGroupWalker f (group);
-    PAN_TREE_STORE(model)->walk (f);
-    setme = f.found_it;
-    return f.found;
-  }
-}
-
 void
-GroupPane :: on_group_subscribe (const Quark& group, bool sub)
+GroupPane :: on_group_subscribe (const Quark& groupname, bool sub)
 {
   GtkTreeModel * model (GTK_TREE_MODEL (_tree_store));
 
-  // remove it from one category
-  Quark * our_group_quark (0);
-  GtkTreeIter iter;
-  if (find_group (model, iter, group)) {
-    our_group_quark = dynamic_cast<Row*>(_tree_store->get_row(&iter))->name;
-    _tree_store->remove (&iter, 1);
-  }
-
-  // add it to the other category
+  // find out where it should be moved to
+  int pos (0);
   GtkTreeIter section_iter, group_iter;
   if (gtk_tree_model_iter_nth_child (model, &section_iter, NULL, (sub?0:1))) {
-    int pos (0);
     if (gtk_tree_model_iter_children (model, &group_iter, &section_iter)) do {
-      gpointer name_g (0);
-      gtk_tree_model_get (model, &group_iter, COL_NAME, &name_g, -1);
-      if (group.to_string() < ((const Quark*)name_g)->to_string())
+      MyRow * row (dynamic_cast<MyRow*>(_tree_store->get_row (&group_iter)));
+      if (groupname.to_string() < row->groupname.c_str())
         break;
       ++pos;
     } while (gtk_tree_model_iter_next (model, &group_iter));
+  }
+  
+  // move the row
+  _tree_store->reparent (_tree_store->get_row(&section_iter), _group_rows[groupname], pos);
 
-    // insert the row in the other category...
-    GtkTreeIter iter;
-    Row * row = create_row (_data, *our_group_quark);
-    pan_tree_store_insert (_tree_store, &iter, &section_iter, row, pos);
-
-    // make sure it's visible...
+  // make sure it's visible...
+  if (sub) {
     GtkTreePath * path = gtk_tree_model_get_path (model, &section_iter);
     gtk_tree_view_expand_row (GTK_TREE_VIEW(_tree_view), path, false);
     gtk_tree_path_free (path);
   }
 }
 
-namespace
+void
+GroupPane :: refresh_dirty_groups ()
 {
-  struct DirtyGroupWalker: public PanTreeStore::WalkFunctor {
-    Data& data;
-    const quarks_t& dirty_groups;
-    DirtyGroupWalker (Data& d, const quarks_t& groups): data(d), dirty_groups(groups) {}
-    virtual ~DirtyGroupWalker () {}
-    virtual bool operator()(PanTreeStore      * store,
-                            PanTreeStore::Row * r,
-                            GtkTreeIter       * it,
-                            GtkTreePath       * unused) {
-      Row * row (dynamic_cast<Row*>(r));
-      if (dirty_groups.count (*row->name)) {
-        data.get_group_counts (*row->name, row->unread, row->total);
-        store->row_changed (it);
-      }
-      return true; // keep going
+  foreach_const (quarks_t, _dirty_groups, it)
+  {
+    group_rows_t::iterator row_it (_group_rows.find (*it));
+    if (row_it != _group_rows.end())
+    {
+      MyRow * row (row_it->second);
+      _data.get_group_counts (row->groupname, row->unread, row->total);
+      _tree_store->row_changed (row);
     }
-  };
+  }
+
+  _dirty_groups_idle_tag = 0;
+  _dirty_groups.clear ();
 }
-
 gboolean
-GroupPane :: dirty_groups_idle (gpointer group_pane_gpointer)
+GroupPane :: dirty_groups_idle (gpointer user_data)
 {
-  GroupPane * self = (GroupPane*) group_pane_gpointer;
-
-  DirtyGroupWalker f (self->_data, self->_dirty_groups);
-  self->_tree_store->walk (f);
-  self->_dirty_groups_idle_tag = 0;
-  self->_dirty_groups.clear ();
-  gtk_widget_queue_draw (self->_tree_view);
+  static_cast<GroupPane*>(user_data)->refresh_dirty_groups ();
   return false;
 }
 
 void
-GroupPane :: on_group_read (const Quark& group)
+GroupPane :: on_group_read (const Quark& groupname)
 {
-  _dirty_groups.insert (group);
+  _dirty_groups.insert (groupname);
+
   if (!_dirty_groups_idle_tag)
-    _dirty_groups_idle_tag = g_idle_add (dirty_groups_idle, this);
+       _dirty_groups_idle_tag = g_timeout_add (333, dirty_groups_idle, this);
 }
 void
-GroupPane :: on_group_counts (const Quark& group, unsigned long unread, unsigned long total)
+GroupPane :: on_group_counts (const Quark& groupname, unsigned long unread, unsigned long total)
 {
-  on_group_read (group);
+  on_group_read (groupname);
 }
 
 /***
@@ -556,7 +483,7 @@ GroupPane :: set_filter (const std::string& search_text)
 
    // build and use the new store
    tree_iters_t iters;
-   PanTreeStore * store = build_model (_data, pmatch, iters, groups_in_model);
+   PanTreeStore * store = build_model (_data, pmatch, iters, _group_rows);
    _tree_store = store;
    gtk_tree_view_set_model(view, GTK_TREE_MODEL(store));
    g_object_unref (G_OBJECT(store));
@@ -687,10 +614,10 @@ namespace
                      gpointer            user_data)
   {
     PanTreeStore * tree (PAN_TREE_STORE(model));
-    Row * row (dynamic_cast<Row*>(tree->get_row (iter)));
+    MyRow * row (dynamic_cast<MyRow*>(tree->get_row (iter)));
     const unsigned long& unread (row->unread);
     //const unsigned long& total (row->total);
-    const Quark& name (*row->name);
+    const Quark& name (row->groupname);
 
     const bool is_g (is_group(name));
     const bool do_shorten (shorten && is_g);
@@ -742,8 +669,8 @@ namespace
     virtual bool operator()(PanTreeStore * store, PanTreeStore::Row * r,
                             GtkTreeIter * iter, GtkTreePath * unused)
     {
-      const Row * row (dynamic_cast<Row*>(r));
-      const Quark& name (*row->name);
+      const MyRow * row (dynamic_cast<MyRow*>(r));
+      const Quark& name (row->groupname);
       if (is_group (name)) {
         if (current_group_reached && (row->unread || !unread_only)) {
           gtk_tree_selection_unselect_all (sel);
@@ -792,14 +719,11 @@ namespace
                gboolean           path_currently_selected,
                gpointer           unused)
   {
-    bool row_can_be_selected (false);
     GtkTreeIter iter;
-    if (gtk_tree_model_get_iter (model, &iter, path))
-    {
-      gpointer name (0);
-      gtk_tree_model_get (model, &iter, COL_NAME, &name, -1);
-      row_can_be_selected = is_group (*(const Quark*)name);
-    }
+    PanTreeStore * store (PAN_TREE_STORE(model));
+    const bool row_can_be_selected =
+      gtk_tree_model_get_iter (model, &iter, path)
+      && is_group (dynamic_cast<MyRow*>(store->get_row (&iter))->groupname);
     return row_can_be_selected;
   }
 }
@@ -861,7 +785,7 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
 
   // build this first because _tree_view is needed for a callback...
   tree_iters_t iters;
-  _tree_store = build_model (_data, NULL, iters, groups_in_model);
+  _tree_store = build_model (_data, NULL, iters, _group_rows);
   _tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(_tree_store));
   g_object_unref (G_OBJECT(_tree_store)); // will die with the view
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW(_tree_view), false);
