@@ -714,6 +714,9 @@ void GUI :: do_read_selected_article ()
     Task * t = new TaskArticle (_data, _data, *article, _cache, _data, this);
     _queue.add_task (t, Queue::TOP);
 
+    // expand its thread in the header pane
+    _header_pane->expand_selected ();
+
     // maybe update the tab
     if (is_action_active ("tabbed-layout"))
       activate_action ("jump-to-body-tab");
