@@ -478,7 +478,7 @@ HeaderPane ::  add_children_to_model (PanTreeStore               * store,
   rows.reserve (children.size());
   foreach_const (article_v, children, it)
     rows.push_back (create_row (date_maker, *it));
-  store->append (parent_row, rows);
+  store->append (do_thread ? parent_row : 0, rows);
 
   // recurse
   for (size_t i=0, n=children.size(); i<n; ++i)
@@ -664,7 +664,9 @@ HeaderPane :: set_group (const Quark& new_group)
 
       rebuild ();
 
-      gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(_tree_view), 0, 0);
+
+      if (GTK_WIDGET_REALIZED(_tree_view))
+        gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(_tree_view), 0, 0);
     }
   }
 
@@ -1809,6 +1811,23 @@ HeaderPane :: select_subthreads ()
   select_threads_helper (GTK_TREE_VIEW(_tree_view), true);
 }
 
+void
+HeaderPane :: expand_selected ()
+{
+  // get a list of paths
+  GtkTreeModel * model (0);
+  GtkTreeView * view (GTK_TREE_VIEW (_tree_view));
+  GtkTreeSelection * selection (gtk_tree_view_get_selection (view));
+  GList * list (gtk_tree_selection_get_selected_rows (selection, &model));
+  if (list)
+  {
+    GtkTreePath * path (static_cast<GtkTreePath*>(list->data));
+    gtk_tree_view_expand_row (view, path, true);
+    gtk_tree_view_expand_to_path (view, path);
+  }
+  g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+  g_list_free (list);
+}
 
 /***
 ****
