@@ -21,8 +21,10 @@
 #include <iostream>
 #include <algorithm>
 #include <cstdarg>
+#include <set>
 #include <gobject/gvaluecollector.h>
 #include <pan/general/debug.h>
+#include <pan/general/foreach.h>
 #include "pan-tree.h"
 
 #define IS_SORTED(tree) \
@@ -72,8 +74,17 @@ PanTreeStore :: Row :: set_value_static_string (GValue * setme, const char * val
 ****  Low-level utilities
 ***/
 
+GtkTreeIter
+PanTreeStore :: get_iter (const Row * row)
+{
+  g_assert (row);
+  GtkTreeIter setme;
+  set_iter (&setme, row);
+  return setme;
+}
+
 void
-PanTreeStore :: get_iter (Row * row, GtkTreeIter * setme)
+PanTreeStore :: get_iter (const Row * row, GtkTreeIter * setme)
 {
   g_assert (setme);
   g_assert (row);
@@ -103,10 +114,10 @@ PanTreeStore :: get_row (const GtkTreeIter * iter) const
 
 void
 PanTreeStore :: set_iter (GtkTreeIter * iter,
-                          Row         * row)
+                          const Row   * row)
 {
   iter->stamp = stamp;
-  iter->user_data = row;
+  iter->user_data = (gpointer)row;
   iter->user_data2 = 0;
   iter->user_data3 = 0;
 }
@@ -149,16 +160,20 @@ PanTreeStore :: model_get_column_type (GtkTreeModel * tree,
 
 gboolean
 PanTreeStore :: model_get_iter (GtkTreeModel * model,
-                                GtkTreeIter  * iter,
+                                GtkTreeIter  * setme,
                                 GtkTreePath  * path)
 {
   PanTreeStore * store = PAN_TREE_STORE(model);
   g_return_val_if_fail (store, false);
   g_return_val_if_fail (path, false);
 
-  // find the Row that corresponds to this path.
-  PanTreeStore::Row * row (store->root);
+  // make sure it's not an empty path.
   const int depth (gtk_tree_path_get_depth (path));
+  if (depth < 1)
+    return false;
+
+  // find the row that correpsonds to this path.
+  PanTreeStore::Row * row (store->root);
   const int * indices = gtk_tree_path_get_indices (path);
   for (int i=0; i<depth; ++i) {
     row = row->nth_child (*indices++);
@@ -167,7 +182,7 @@ PanTreeStore :: model_get_iter (GtkTreeModel * model,
   }
 
   // build an iter from that row.
-  store->set_iter (iter, row);
+  store->set_iter (setme, row);
   return true;
 }
 
@@ -313,22 +328,44 @@ PanTreeStore :: get_parent (GtkTreeIter * iter,
   g_return_val_if_fail (child->stamp == stamp, false);
 
   const Row * row (get_row (child));
-  const bool has_parent = row->parent != root;
-  if (has_parent)
+  const bool has_public_parent (row->parent != root);
+  if (has_public_parent)
     set_iter (iter, row->parent);
   else
     invalidate_iter (iter);
-  return has_parent;
+  return has_public_parent;
 }
 
 bool
 PanTreeStore :: is_root (const GtkTreeIter* iter) const
 {
-  return iter && get_row(iter)->parent==root;
+  g_return_val_if_fail (iter, false);
+  g_return_val_if_fail (iter->stamp == stamp, false);
+  return get_row(iter)->parent == root;
 }
 
+size_t
+PanTreeStore :: get_depth (const Row * row) const
+{
+  g_assert (row);
+  size_t depth (0);
+  for (;;) {
+    if (row == root) break;
+    ++depth;
+    row = row->parent;
+    g_assert (row);
+  }
+  return depth;
+}
 
-
+bool
+PanTreeStore :: is_in_tree (Row * row) const
+{
+  while (row && row!=root)
+    row = row->parent;
+  return row == root;
+}
+  
 /*****
 ******
 ******  implementing GtkTreeSortable
@@ -337,16 +374,16 @@ PanTreeStore :: is_root (const GtkTreeIter* iter) const
 
 gboolean
 PanTreeStore :: sortable_get_sort_column_id (GtkTreeSortable  * sortable,
-                                             gint             * sort_column_id,
-                                             GtkSortType      * order)
+                                             gint             * setme_column_id,
+                                             GtkSortType      * setme_order)
 {
   PanTreeStore * tree (PAN_TREE_STORE (sortable));
   g_return_val_if_fail (tree, false);
 
-  if (sort_column_id)
-     *sort_column_id = tree->sort_column_id;
-  if (order)
-     *order = tree->order;
+  if (setme_column_id)
+     *setme_column_id = tree->sort_column_id;
+  if (setme_order)
+     *setme_order = tree->order;
 
   return (tree->sort_column_id != GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
       && (tree->sort_column_id != GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID);
@@ -355,11 +392,10 @@ PanTreeStore :: sortable_get_sort_column_id (GtkTreeSortable  * sortable,
 gboolean
 PanTreeStore :: sortable_has_sort_func (GtkTreeSortable *sortable, gint col)
 {
-  PanTreeStore * tree (PAN_TREE_STORE (sortable));
-  g_return_val_if_fail (tree, false);
-  std::map<int,PanTreeStore::SortInfo>::const_iterator it =
-    tree->sort_info->find (col);
-  return (it!=tree->sort_info->end()) && (it->second.sort_func!=0);
+  PanTreeStore * store (PAN_TREE_STORE (sortable));
+  g_return_val_if_fail (store, false);
+  column_sort_info_t::const_iterator it (store->sort_info->find (col));
+  return (it!=store->sort_info->end()) && (it->second.sort_func!=0);
 }
 
 // SORTING
@@ -372,15 +408,15 @@ PanTreeStore :: SortRowInfo
   GtkTreeIter iter;
 };
 
-struct PanTreeStore :: SortHelperData
+struct PanTreeStore :: SortData
 {
   PanTreeStore * store;
   GtkTreeModel * model;
   PanTreeStore::SortInfo& sort_info;
   GtkSortType order;
-  SortHelperData (PanTreeStore           * tree_store,
-                  PanTreeStore::SortInfo & s,
-                  GtkSortType              o):
+  SortData (PanTreeStore           * tree_store,
+            PanTreeStore::SortInfo & s,
+            GtkSortType              o):
     store(tree_store), model(GTK_TREE_MODEL(tree_store)), sort_info(s), order(o) {}
 };
 
@@ -391,17 +427,17 @@ PanTreeStore :: row_compare_func (gconstpointer a_gpointer,
 {
   const SortRowInfo * a = (const SortRowInfo*) a_gpointer;
   const SortRowInfo * b = (const SortRowInfo*) b_gpointer;
-  const SortHelperData * help = (const SortHelperData*) user_data;
+  const SortData * help = (const SortData*) user_data;
   int val = (help->sort_info.sort_func) (help->model,
                                          const_cast<GtkTreeIter*>(&a->iter),
                                          const_cast<GtkTreeIter*>(&b->iter),
                                          help->sort_info.user_data);
-  if (help->order == GTK_SORT_DESCENDING)
-    val = -val;
-
   if (!val) // inplace sort
     val = a->row->child_index - b->row->child_index;
 
+  if (help->order == GTK_SORT_DESCENDING)
+    val = -val;
+
   return val;
 }
 
@@ -413,7 +449,7 @@ PanTreeStore :: sort_children (SortInfo  & sort_info,
   g_assert (parent);
 
   const int n (parent->n_children());
-  if (n < 2) // no need to sort [0...1] children
+  if (n < 2) // no need to sort
     return;
 
   // build a temporary array to sort
@@ -426,44 +462,45 @@ PanTreeStore :: sort_children (SortInfo  & sort_info,
   }
 
   // sort 'em...
-  SortHelperData help (this, sort_info, order);
-  g_qsort_with_data (sort_array, n, sizeof(SortRowInfo),
+  SortData help (this, sort_info, order);
+  g_qsort_with_data (sort_array,
+                     n, sizeof(SortRowInfo),
                      row_compare_func, &help);
 
   // update the child indices...
+  bool reordered (false);
   for (int i=0; i<n; ++i) {
     Row * child = sort_array[i].row;
+    reordered |= (child->child_index != i);
     child->child_index = i;
     parent->children[i] = child;
     g_assert (child->parent == parent);
   }
 
   // let the world know we've changed
-  int * new_order (new int [n]);
-  bool reordered (false);
-  for (int i=0; i<n; ++i) {
-    reordered |= (i != sort_array[i].oldpos);
-    new_order[i] = sort_array[i].oldpos;
-  }
+  if (reordered)
+  {
+    int * new_order (new int [n]);
+    for (int i=0; i<n; ++i)
+      new_order[i] = sort_array[i].oldpos;
 
-  // tell the world...
-  if (reordered) {
-    GtkTreeIter *parent_iter;
-    GtkTreePath * path;
-    if (parent == root) {
-      parent_iter = 0;
-      path = gtk_tree_path_new ();
-    } else {
-      GtkTreeIter tmp_iter;
-      set_iter (&tmp_iter, parent);
-      parent_iter = &tmp_iter;
-      path = gtk_tree_model_get_path (help.model, parent_iter);
+    if (parent == root)
+    {
+      GtkTreePath * path (gtk_tree_path_new ());
+      gtk_tree_model_rows_reordered (help.model, path, 0, new_order);
+      gtk_tree_path_free (path);
     }
-    gtk_tree_model_rows_reordered (help.model, path, parent_iter, new_order);
-    gtk_tree_path_free (path);
+    else
+    {
+      GtkTreeIter it (get_iter (parent));
+      GtkTreePath * path (get_path (parent));
+      gtk_tree_model_rows_reordered (help.model, path, &it, new_order);
+      gtk_tree_path_free (path);
+    }
+
+    delete [] new_order;
   }
 
-  delete [] new_order;
   delete [] sort_array;
 
   for (int i=0; recurse && i<n; ++i)
@@ -476,8 +513,7 @@ PanTreeStore :: sort ()
   if (!sort_paused && (sort_column_id != GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID))
   {
     g_assert (sortable_has_sort_func (GTK_TREE_SORTABLE(this), sort_column_id));
-    SortInfo& info = (*sort_info)[sort_column_id];
-    sort_children (info, root, true);
+    sort_children ((*sort_info)[sort_column_id], root, true);
   }
 }
 
@@ -624,7 +660,7 @@ PanTreeStore :: pan_tree_init (PanTreeStore * tree)
   tree->root = new RootRow ();
   tree->column_types = new std::vector<GType>();
   tree->sort_paused = 0;
-  tree->sort_info = new std::map<int,PanTreeStore::SortInfo>();
+  tree->sort_info = new column_sort_info_t;
   tree->sort_column_id = GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID;
   tree->order = GTK_SORT_ASCENDING;
 }
@@ -674,7 +710,7 @@ PanTreeStore :: new_tree (int n_columns, ...)
   va_list args;
   va_start (args, n_columns);
   for (int i=0; i<n_columns; ++i) {
-    const GType type = va_arg (args, GType);
+    const GType type (va_arg (args, GType));
     tree->column_types->push_back (type);
   }
   va_end (args);
@@ -698,8 +734,8 @@ PanTreeStore :: row_changed (Row * row)
 void
 PanTreeStore :: row_changed (GtkTreeIter * iter)
 {
-  GtkTreeModel * model = GTK_TREE_MODEL(this);
-  GtkTreePath * path = gtk_tree_model_get_path (model, iter);
+  GtkTreeModel * model (GTK_TREE_MODEL(this));
+  GtkTreePath * path (gtk_tree_model_get_path (model, iter));
   gtk_tree_model_row_changed (model, path, iter);
   gtk_tree_path_free (path);
 }
@@ -710,8 +746,8 @@ PanTreeStore :: row_changed (GtkTreeIter * iter)
 
 void
 PanTreeStore :: renumber_children (Row * parent,
-                                   int child_lo,
-                                   int child_hi)
+                                   int   child_lo,
+                                   int   child_hi)
 {
   const int n_children (parent->n_children());
   child_hi = std::min (child_hi, n_children);
@@ -728,101 +764,85 @@ PanTreeStore :: renumber_children (Row * parent,
   }
 }
 
-typedef std::vector<PanTreeStore::Row*> rows_t;
+struct PanTreeStore :: RowCompareByDepth
+{
+  PanTreeStore * store;
+  RowCompareByDepth (PanTreeStore *s): store(s) {}
+  bool operator() (const Row* a, const Row* b) const {
+    return store->get_depth(a) < store->get_depth(b);
+  }
+};
 
 void
-pan_tree_store_insert     (PanTreeStore                     * tree,
-                           GtkTreeIter                      * iter,
-                           GtkTreeIter                      * parent,
-                           PanTreeStore::Row                * row,
-                           int                                pos)
+PanTreeStore :: insert_sorted (const parent_to_children_t& new_parents)
 {
-  rows_t rows;
-  rows.push_back (row);
-  pan_tree_store_insert_n (tree, iter, parent, rows, pos);
+  g_return_if_fail (!new_parents.empty());
+
+  RowCompareByDepth depth_compare (this);
+
+  // this is a pool of all the parents that need kids added...
+  std::set<Row*> pool;
+  foreach_const (parent_to_children_t, new_parents, it)
+    pool.insert (it->first ? it->first : root);
+
+  while (!pool.empty())
+  {
+    // get a subset of pool: parents that are are in the tree
+    rows_t keys;
+    foreach_const (std::set<Row*>, pool, it)
+      if (is_in_tree (*it))
+        keys.push_back (*it);
+    g_assert (!keys.empty());
+
+    // sort these from shallowest to deepest
+    std::sort (keys.begin(), keys.end(), depth_compare);
+ 
+    // process each of them
+    foreach_const (rows_t, keys, it)
+    {
+      // add the children to this parent...
+      // caller passes in NULL for the parent of top-level articles...
+      Row * parent (*it);
+      Row * key (parent==root ? 0 : parent);
+      const rows_t& children (new_parents.find(key)->second);
+      insert_sorted (parent, children);
+      pool.erase (parent);
+    }
+  }
 }
 
-void
-pan_tree_store_insert_n  (PanTreeStore                    * tree,
-                          GtkTreeIter                     * iter,
-                          GtkTreeIter                     * parent_it,
-                          rows_t                          & rows,
-                          int                               position)
+struct PanTreeStore :: RowCompareByColumn
 {
-  tree->insert_n (iter, parent_it, rows, position);
-}
+  PanTreeStore * store;
+  GtkTreeModel * model;
+  GtkTreeIterCompareFunc sort_func;
+  gpointer user_data;
+  bool reverse;
 
-void
-PanTreeStore :: insert_n (GtkTreeIter                     * iter,
-                          GtkTreeIter                     * parent_it,
-                          rows_t                          & new_children,
-                          int                               position)
-{
-  g_return_if_fail (!new_children.empty());
-  g_return_if_fail (position >= 0);
-
-  // insert the rows
-  const size_t n_rows (new_children.size());
-  Row * parent_row (parent_it ? get_row (parent_it) : root);
-  const int old_size (parent_row->n_children());
-  position = std::min (position, old_size);
-  parent_row->children.insert (parent_row->children.begin()+position,
-                               new_children.begin(),
-                               new_children.end());
-  renumber_children (parent_row, position);
-
-  // set the return iter
-  GtkTreeIter walk;
-  set_iter (&walk, parent_row->children[position]);
-  if (iter)
-    *iter = walk;
-
-  // emit the 'row inserted' signals
-  GtkTreePath * path = get_path (&walk);
-  GtkTreeModel * model = GTK_TREE_MODEL(this);
-  Row ** it = &parent_row->children[position];
-  for (size_t i=0; i<n_rows; ++i) {
-    Row * row (*it++);
-    set_iter (&walk, row);
-    gtk_tree_model_row_inserted (model, path, &walk);
-    gtk_tree_path_next (path);
+  RowCompareByColumn (PanTreeStore * s):
+    store (s),
+    model (GTK_TREE_MODEL(s))
+  {
+    const SortInfo& info ((*(store->sort_info))[store->sort_column_id]);
+    sort_func = info.sort_func;
+    user_data = info.user_data;
+    reverse = (store->order == GTK_SORT_DESCENDING);
   }
 
-  // maybe emit the 'row has child toggled' signal
-  if (!old_size && parent_row!=root) {
-    gtk_tree_path_up (path);
-    gtk_tree_model_row_has_child_toggled (model, path, parent_it);
+  bool operator() (const Row* a, const Row* b) const
+  {
+    GtkTreeIter a_it (store->get_iter (a));
+    GtkTreeIter b_it (store->get_iter (b));
+    int val = (sort_func)(model, &a_it, &b_it, user_data);
+    if (reverse) val = -val;
+    return val < 0;
   }
-
-  // cleanup
-  gtk_tree_path_free (path);
-}
-
-void
-pan_tree_store_append  (PanTreeStore       * tree,
-                        GtkTreeIter        * iter,
-                        GtkTreeIter        * parent,
-                        PanTreeStore::Row  * row)
-{
-  rows_t rows;
-  rows.push_back (row);
-  pan_tree_store_append_n (tree, iter, parent, rows);
-}
-
-void
-pan_tree_store_append_n  (PanTreeStore  * tree,
-                          GtkTreeIter   * iter,
-                          GtkTreeIter   * parent,
-                          rows_t        & rows)
-{
-  pan_tree_store_insert_n (tree, iter, parent, rows, INT_MAX);
-}
+};
 
 /***
 ****
 ***/
 
-
 struct PanTreeStore::ReparentWalker: public PanTreeStore::WalkFunctor
 {
   GtkTreeModel * model;
@@ -857,6 +877,165 @@ struct PanTreeStore::ReparentWalker: public PanTreeStore::WalkFunctor
   }
 };
 
+void
+PanTreeStore :: insert_sorted (Row           * parent,
+                               const rows_t  & children_in)
+{
+  g_return_if_fail (!children_in.empty());
+
+  if (!parent)
+       parent = root;
+
+//std::cerr << LINE_ID << " adding " << children_in.size() << " sorted children to " << parent << " which currently has " << parent->children.size() << " children" << std::endl;
+
+  // sort new children...
+  rows_t children (children_in);
+  foreach (rows_t, children, it)
+    (*it)->parent = parent;
+  RowCompareByColumn compare_column (this);
+  std::sort (children.begin(), children.end(), compare_column);
+
+//for (int i=0, n=children.size()-1; i<n; ++i)
+ // g_assert (!compare_column (children[i+1], children[i]));
+//for (int i=0, n=parent->children.size()-1; i<n; ++i)
+ // g_assert (!compare_column (parent->children[i+1], parent->children[i]));
+
+
+
+  // merge the two sorted containers together...
+  const size_t old_size (parent->children.size());
+  rows_t tmp;
+  tmp.reserve (old_size + children.size());
+  const size_t n_new (children.size());
+  std::vector<int> indices;
+  indices.reserve (n_new);
+  rows_t::const_iterator o_it  (parent->children.begin()),
+                         o_end (parent->children.end()),
+                         n_it  (children.begin()),
+                         n_end (children.end());
+  for (size_t i=0; o_it!=o_end || n_it!=n_end; ++i) {
+    Row * addme (0);
+    if ((n_it==n_end)) {
+      //std::cerr << LINE_ID << " n is empty ... adding another o at " << tmp.size() << std::endl;
+      addme = *o_it++;
+    }
+    else if (o_it==o_end) {
+      //std::cerr << LINE_ID << " o is empty .. adding NEW at " << tmp.size() << std::endl;
+      indices.push_back (i);
+      addme = *n_it++;
+    }
+    else if (compare_column (*n_it, *o_it)) {
+      //std::cerr << LINE_ID << " adding NEW at index " << tmp.size() << std::endl;
+      indices.push_back (i);
+      addme = *n_it++;
+    }
+    else {
+      //std::cerr << LINE_ID << " adding old at index " << tmp.size() << std::endl;
+      addme = *o_it++;
+    }
+  //  g_assert (tmp.empty() || !compare_column(addme,tmp.back()));
+    addme->child_index = i;
+   // g_assert (tmp.empty() || !compare_column(addme,tmp.back()));
+    tmp.push_back (addme);
+  }
+  parent->children.swap (tmp);
+
+  // check our work...
+  //g_assert (parent->children.size() == old_size + children.size());
+  //g_assert (indices.size() == children.size());
+  //for (int i=0, n=parent->children.size(); i<n; ++i)
+   // g_assert (i==0 || !compare_column(parent->children[i], parent->children[i-1]));
+
+  // emit the 'row inserted' signals
+  GtkTreePath * path (get_path (parent));
+  GtkTreeIter iter;
+  GtkTreeModel * model (GTK_TREE_MODEL (this));
+  foreach_const (std::vector<int>, indices, it)
+  {
+    Row * child (parent->nth_child (*it));
+    get_iter (child, &iter);
+    gtk_tree_path_append_index (path, *it);
+    gtk_tree_model_row_inserted (model, path, &iter);
+    gtk_tree_path_up (path);
+
+    // if row has children, this handles all of their
+    // row-inserted and has-child-toggled signals...
+    // if the row doesn't have children, this has
+    // no effect...
+    if (!child->children.empty()) {
+      ReparentWalker walk (this);
+      prefix_walk (walk, &iter, true);
+    }
+  }
+
+  // maybe emit the 'row has child toggled' signal
+  if (!old_size && parent!=root) {
+    GtkTreeIter it (get_iter (parent));
+    gtk_tree_model_row_has_child_toggled (model, path, &it);
+  }
+
+  // cleanup
+  gtk_tree_path_free (path);
+
+  //for (int i=0, n=parent->children.size(); i<n; ++i)
+   // g_assert (i==0 || !compare_column(parent->children[i], parent->children[i-1]));
+}
+
+void
+PanTreeStore :: insert (Row            * parent_row,
+                        const rows_t   & new_children,
+                        int              position)
+{
+  g_return_if_fail (!new_children.empty());
+  g_return_if_fail (position >= 0);
+
+  if (!parent_row)
+       parent_row = root;
+
+  // insert the rows
+  const size_t n_rows (new_children.size());
+  const int old_size (parent_row->n_children());
+  position = std::min (position, old_size);
+  parent_row->children.insert (parent_row->children.begin()+position,
+                               new_children.begin(),
+                               new_children.end());
+  renumber_children (parent_row, position);
+
+  // set the return iter
+  GtkTreeIter iter (get_iter (parent_row->children[position]));
+
+  // emit the 'row inserted' signals
+  GtkTreeModel * model (GTK_TREE_MODEL(this));
+  Row ** it (&parent_row->children[position]);
+  GtkTreePath * path (get_path (*it));
+  for (size_t i=0; i<n_rows; ++i)
+  {
+    Row * child (*it++);
+    set_iter (&iter, child);
+    gtk_tree_model_row_inserted (model, path, &iter);
+    gtk_tree_path_next (path);
+
+    // if row has children, this handles all of their
+    // row-inserted and has-child-toggled signals...
+    // if the row doesn't have children, this has
+    // no effect...
+    if (!child->children.empty()) {
+      ReparentWalker walk (this);
+      prefix_walk (walk, &iter, true);
+    }
+  }
+
+  // maybe emit the 'row has child toggled' signal
+  if (!old_size && parent_row!=root) {
+    GtkTreeIter parent_it (get_iter (parent_row));
+    gtk_tree_path_up (path);
+    gtk_tree_model_row_has_child_toggled (model, path, &parent_it);
+  }
+
+  // cleanup
+  gtk_tree_path_free (path);
+}
+
 void
 PanTreeStore :: reparent (Row  * new_parent,
                           Row  * row,
@@ -866,22 +1045,15 @@ PanTreeStore :: reparent (Row  * new_parent,
 
   GtkTreeModel * model (GTK_TREE_MODEL(this));
 
-  Row * old_parent (row->parent);
+  //Row * old_parent (row->parent);
   if (!new_parent)
     new_parent = root;
   const int new_parent_old_n_children (new_parent->n_children());
 
   // remove our subtree's toplevel from its old parent
-  remove_siblings (&row, 1, false);
-
-  // if this was the old parent's last child, fire a has-child-toggled event
-  if (!old_parent->n_children()) {
-    GtkTreeIter tmp;
-    set_iter (&tmp, old_parent);
-    GtkTreePath * path (get_path (&tmp));
-    gtk_tree_model_row_has_child_toggled (model, path, &tmp);
-    gtk_tree_path_free (path);
-  }
+  rows_t tmp;
+  tmp.push_back (row);
+  remove_siblings (tmp, false);
 
   // add the subtree's toplevel to its new parent row
   position = std::min (position, new_parent_old_n_children);
@@ -889,9 +1061,8 @@ PanTreeStore :: reparent (Row  * new_parent,
   renumber_children (new_parent, position);
 
   // emit a row-inserted for iter
-  GtkTreeIter iter;
-  get_iter (row, &iter);
-  GtkTreePath * path (get_path (&iter));
+  GtkTreeIter iter (get_iter (row));
+  GtkTreePath * path (get_path (row));
   gtk_tree_model_row_inserted (model, path, &iter);
   gtk_tree_path_free (path);
 
@@ -911,77 +1082,46 @@ PanTreeStore :: reparent (Row  * new_parent,
   }
 }
 
+void
+PanTreeStore :: reparent (const parent_to_children_t& new_parents)
+{
+  const RowCompareByDepth depth_compare (this);
 
+  // REMOVE children from their OLD parents
+
+  rows_t remove_me;
+  foreach_const (parent_to_children_t, new_parents, it)
+    remove_me.insert (remove_me.end(), it->second.begin(), it->second.end());
+  remove (remove_me, false);
+
+  // ADD children to their NEW parent
+
+  insert_sorted (new_parents);
+}
+  
 /***
 ****
 ***/
 
-struct PanTreeStore :: RowCompare
-{
-  Row * root;
-  RowCompare (Row * r): root(r) {}
-
-  int depth (const Row * row) const {
-    int depth (0);
-    while (row->parent != root) {
-      ++depth;
-      row = row->parent;
-    }
-    return depth;
-  }
-
-  // this sort function is very particular...
-  // we go deepest first, so the first batches don't invalidate later ones.
-  // we group by parent so to get groups of siblings for remove_siblings().
-  // we sort by child_index so the first siblings removed in
-  // remove_siblings() don't invalidate the tree path it uses.
-  bool operator() (const Row* a, const Row* b) const {
-    // deepest first
-    const int a_depth (depth (a));
-    const int b_depth (depth (b));
-    if (a_depth != b_depth)
-      return b_depth < a_depth;
-    // group by parent
-    if (a->parent != b->parent)
-      return a->parent < b->parent;
-    // from last child to first
-    return b->child_index < a->child_index;
-  }
-};
-
 void
-PanTreeStore :: remove (GtkTreeIter * iters,
-                        size_t        n_iters)
+PanTreeStore :: remove (const rows_t& rows, bool delete_rows)
 {
-  g_return_if_fail (iters);
-  g_return_if_fail (n_iters);
-
-  Row ** rows = new Row* [n_iters];
-  for (size_t i=0; i<n_iters; ++i)
-    rows[i] = get_row (iters+i);
-
-  RowCompare compare (root);
-  std::sort (rows, rows+n_iters, compare);
-
-  Row * parent (0);
-  Row ** children (0);
-  for (Row **nit=rows, **nend=rows+n_iters; nit!=nend; ++nit) {
-    if (parent && ((*nit)->parent != parent)) {
-      remove_siblings (children, nit-children, true);
-      parent = 0;
-    }
-    if (!parent) {
-      parent = (*nit)->parent;
-      g_assert (parent);
-      children = nit;
-    }
-  }
-  if (parent) {
-    size_t n = (rows + n_iters) - children;
-    remove_siblings (children, n, true);
+  parent_to_children_t parent_to_kids;
+  foreach_const (rows_t, rows, it) {
+    g_assert (*it);
+    g_assert ((*it)->parent);
+    parent_to_kids[(*it)->parent].push_back (*it);
   }
 
-  delete [] rows;
+  // remove them in same-parent batches, starting at the
+  // deepest parents and working up to the shallower ones
+  rows_t keys;
+  foreach_const (parent_to_children_t, parent_to_kids, it)
+    keys.push_back (it->first);
+  const RowCompareByDepth compare_depth (this);
+  std::sort (keys.begin(), keys.end(), compare_depth);
+  foreach_const_r (rows_t, keys, it)
+    remove_siblings (parent_to_kids[*it], delete_rows);
 }
 
 struct PanTreeStore::ClearWalker: public PanTreeStore::WalkFunctor
@@ -990,11 +1130,7 @@ struct PanTreeStore::ClearWalker: public PanTreeStore::WalkFunctor
 
   void clear_children (PanTreeStore* store, Row * row)
   {
-    if (row->n_children()) {
-      std::vector<Row*> children (row->children);
-      std::reverse (children.begin(), children.end());
-      store->remove_siblings (&children.front(), children.size(), true);
-    }
+    store->remove_siblings (row->children, true);
   }
 
   virtual bool operator()(PanTreeStore* store,
@@ -1010,58 +1146,76 @@ void
 PanTreeStore :: clear ()
 {
   ClearWalker walker;
-  postfix_walk (walker, 0, false);
+  postfix_walk (walker);
   walker.clear_children (this, root);
 }
 
 
+struct PanTreeStore :: RowCompareByChildPos {
+  bool operator() (const Row* a, const Row* b) const {
+    return a->child_index < b->child_index;
+  }
+};
+
 // note that the siblings passed in here must
 // be sorted from highest child index to lowest child index
 void
-PanTreeStore :: remove_siblings (Row     ** siblings,
-                                 size_t     n_siblings,
-                                 bool       free_memory)
+PanTreeStore :: remove_siblings (const rows_t& siblings_in,
+                                 bool          delete_rows)
 {
+  RowCompareByChildPos compare_index;
+  rows_t siblings (siblings_in);
+  std::sort (siblings.begin(), siblings.end(), compare_index);
+  
   // entry assertions
-  g_assert (siblings);
-  g_assert (n_siblings);
-  Row * parent = siblings[0]->parent;
-  for (Row **nit(siblings), **nend(nit+n_siblings); nit!=nend; ++nit) {
-    g_assert ((*nit)->parent == parent); // all are siblings
-    g_assert (nit==siblings || ((*nit)->child_index < nit[-1]->child_index)); // sorted from high to low child index
-  }
+  g_assert (!siblings.empty());
+  Row * parent (siblings[0]->parent);
+  g_assert (parent);
+  foreach_const (rows_t, siblings, it)
+    g_assert ((*it)->parent == parent); // all are siblings
 
   // walk through all the siblings and unthread them.
+  // NOTE: we walk through from high child_index to low
+  // so that subsequent removals won't mess up the
+  // indices that we place into "removed_indices"
+  std::vector<int> removed_indices;
+  removed_indices.reserve (siblings.size());
   GtkTreeModel * model (GTK_TREE_MODEL(this));
-  GtkTreePath * path = get_path (parent);
-  for (Row** nit(siblings), **nend(siblings+n_siblings); nit!=nend; ++nit)
+  GtkTreePath * path (get_path (parent));
+  foreach_const_r (rows_t, siblings, nit)
   {
+    // this sibling...
     Row * row (*nit);
-    g_assert (row->parent);
-    g_assert (row->parent == parent);
+    //g_assert (row->parent);
+    //g_assert (row->parent == parent);
 
-    // unthread the row
-    const int pos = row->child_index;
+    // unlink this row
+    const int pos (row->child_index);
+    removed_indices.push_back (pos);
+    row->parent = 0;
+    row->child_index = -1;
     parent->children.erase (parent->children.begin() + pos);
 
-    // let the world know the row was removed
-    gtk_tree_path_append_index (path, row->child_index);
-    gtk_tree_model_row_deleted (model, path);
-    gtk_tree_path_up (path);
-
     // free `row' and all its descendants
-    if (free_memory) {
+    if (delete_rows) {
       FreeRowWalker walk (this);
-      GtkTreeIter iter;
-      set_iter (&iter, *nit);
+      GtkTreeIter iter (get_iter (row));
       postfix_walk (walk, &iter);
     }
   }
 
   // renumber the surviving children
-  renumber_children (parent);
+  std::sort (removed_indices.begin(), removed_indices.end());
+  renumber_children (parent, removed_indices.front());
 
-  // if the parent's has-children state has changed, let the world know that too
+  // fire removal signal for child
+  foreach_const_r (std::vector<int>, removed_indices, it) {
+    gtk_tree_path_append_index (path, *it);
+    gtk_tree_model_row_deleted (model, path);
+    gtk_tree_path_up (path);
+  }
+
+  // maybe fire has-children signal for parent
   if (parent!=root && !parent->n_children()) {
     GtkTreeIter pit;
     set_iter (&pit, parent);
