@@ -1754,7 +1754,8 @@ PostUI :: ~PostUI ()
   g_object_unref (G_OBJECT(_message));
 }
 
-PostUI :: PostUI (Data         & data,
+PostUI :: PostUI (GtkWindow    * parent,
+                  Data         & data,
                   Queue        & queue,
                   GroupServer  & gs,
                   Profiles     & profiles,
@@ -1781,6 +1782,7 @@ PostUI :: PostUI (Data         & data,
 {
   GtkTooltips * tips = gtk_tooltips_new ();
 
+  g_assert (profiles.has_profiles());
   g_return_if_fail (message != 0);
 
   // create the window
@@ -1789,6 +1791,10 @@ PostUI :: PostUI (Data         & data,
   gtk_window_set_role (GTK_WINDOW(_root), "pan-post-window");
   gtk_window_set_title (GTK_WINDOW(_root), _("Post Article"));
   g_object_set_data_full (G_OBJECT(_root), "post-ui", this, delete_post_ui);
+  if (parent) {
+    gtk_window_set_transient_for (GTK_WINDOW(_root), parent);
+    gtk_window_set_position (GTK_WINDOW(_root), GTK_WIN_POS_CENTER_ON_PARENT);
+  }
 
   // populate the window
   GtkWidget * vbox = gtk_vbox_new (false, PAD_SMALL);
@@ -1858,23 +1864,6 @@ PostUI :: PostUI (Data         & data,
   g_signal_connect (_body_buf, "insert-text", G_CALLBACK(text_was_inserted_cb), &_tm);
   gtk_box_pack_start (GTK_BOX(vbox), w, true, true, 0);
 
-  // ensure that we've got at least one profile to choose from...
-  gtk_widget_show_all (vbox);
-  if (!_profiles.has_profiles()) {
-    gtk_widget_show_all (_root);
-    Profile profile;
-    profile.username = g_get_real_name ();
-    profile.attribution = _("On %d, %n wrote:");
-    ProfileDialog d (_data, g_get_real_name(), profile, GTK_WINDOW(_root));
-    ProfileDialog :: run_until_valid (d);
-    std::string name;
-    d.get_profile (name, profile);
-    _profiles.add_profile (name, profile);
-    gtk_widget_destroy (d.root());
-  }
-
-  gtk_window_set_position (GTK_WINDOW(_root), GTK_WIN_POS_CENTER_ALWAYS);
-
   set_message (message);
 
   _unchanged_body = get_text (_body_buf);
@@ -1896,3 +1885,34 @@ PostUI :: PostUI (Data         & data,
     grab = _body_view;
   gtk_widget_grab_focus (grab);
 }
+
+PostUI*
+PostUI :: create_window (GtkWindow    * parent,
+                         Data         & data,
+                         Queue        & queue,
+                         GroupServer  & gs,
+                         Profiles     & profiles,
+                         GMimeMessage * message,
+                         Prefs        & prefs,
+                         GroupPrefs   & group_prefs)
+{
+  // can't post without a profile...
+  if (!profiles.has_profiles())
+  {
+    Profile profile;
+    profile.username = g_get_real_name ();
+    profile.attribution = _("On %d, %n wrote:");
+    ProfileDialog d (data, g_get_real_name(), profile, GTK_WINDOW(parent));
+    const bool got_profile (ProfileDialog :: run_until_valid_or_cancel (d));
+    if (got_profile) {
+      std::string name;
+      d.get_profile (name, profile);
+      profiles.add_profile (name, profile);
+    }
+    gtk_widget_destroy (d.root());
+    if (!got_profile)
+      return 0;
+  }
+
+  return new PostUI (parent, data, queue, gs, profiles, message, prefs, group_prefs);
+}
