@@ -409,16 +409,21 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
     HIG :: workarea_add_wide_control (t, &row, w);
   HIG::workarea_add_section_divider (t, &row);
   HIG::workarea_add_section_title (t, &row, _("Groups"));
-    HIG::workarea_add_section_spacer (t, row, 3);
+    HIG::workarea_add_section_spacer (t, row, 4);
     w = new_check_button (_("Get new headers in subscribed groups on _startup"), "get-new-headers-on-startup", false, prefs);
     HIG :: workarea_add_wide_control (t, &row, w);
     w = new_check_button (_("Get new headers when _entering group"), "get-new-headers-when-entering-group", true, prefs);
     HIG :: workarea_add_wide_control (t, &row, w);
     w = new_check_button (_("Mark entire group _read when leaving group"), "mark-group-read-when-leaving-group", false, prefs);
     HIG :: workarea_add_wide_control (t, &row, w);
-    HIG :: workarea_add_section_spacer (t, row, 1);
-    HIG :: workarea_add_section_title (t, &row, _("Articles"));
-    w = new_check_button (_("Space selects next article, not next unread"), "space-selects-next-article", true, prefs);
+    w = new_check_button (_("E_xpand all threads when entering group"), "expand-threads-when-entering-group", false, prefs);
+    HIG :: workarea_add_wide_control (t, &row, w);
+  HIG::workarea_add_section_divider (t, &row);
+  HIG :: workarea_add_section_title (t, &row, _("Articles"));
+    HIG :: workarea_add_section_spacer (t, row, 2);
+    w = new_check_button (_("Space selects next article rather than next unread"), "space-selects-next-article", true, prefs);
+    HIG :: workarea_add_wide_control (t, &row, w);
+    w = new_check_button (_("Smooth scrolling"), "smooth-scrolling", true, prefs);
     HIG :: workarea_add_wide_control (t, &row, w);
   HIG :: workarea_finish (t, &row);
   gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, gtk_label_new_with_mnemonic(_("_Behavior")));
@@ -464,11 +469,6 @@ PrefsDialog :: PrefsDialog (Prefs& prefs, GtkWindow* parent):
   HIG :: workarea_add_section_title (t, &row, _("Header Pane Columns"));
     HIG :: workarea_add_section_spacer(t, row, 1);
     HIG :: workarea_add_wide_control (t, &row, header_columns_layout_new (prefs));
-  HIG :: workarea_add_section_divider (t, &row);
-  HIG::workarea_add_section_title (t, &row, _("Threads"));
-    HIG::workarea_add_section_spacer (t, row, 1);
-    w = new_check_button (_("E_xpand all threads when entering group"), "expand-threads-when-entering-group", false, prefs);
-    HIG :: workarea_add_wide_control (t, &row, w);
   HIG :: workarea_finish (t, &row);
   gtk_notebook_append_page (GTK_NOTEBOOK(notebook), t, gtk_label_new_with_mnemonic(_("_Headers")));
 
