@@ -101,8 +101,10 @@ ProfileDialog :: ProfileDialog (const Data         & data,
 
   _root = gtk_dialog_new_with_buttons (_("Posting Profile"), parent,
                                        GTK_DIALOG_DESTROY_WITH_PARENT,
-                                       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+                                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                       GTK_STOCK_APPLY, GTK_RESPONSE_OK,
                                        NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG(_root), GTK_RESPONSE_OK); 
   gtk_window_set_role (GTK_WINDOW(_root), "pan-edit-profile-dialog");
 
   int row (0);
@@ -230,12 +232,15 @@ namespace
   }
 }
 
-/*static*/ void
-ProfileDialog :: run_until_valid (ProfileDialog& pd)
+/*static*/ bool
+ProfileDialog :: run_until_valid_or_cancel (ProfileDialog& pd)
 {
   for (;;)
   {
-    gtk_dialog_run (GTK_DIALOG(pd.root()));
+    const int response (gtk_dialog_run (GTK_DIALOG(pd.root())));
+
+    if (response == GTK_RESPONSE_CANCEL)
+      return false;
 
     std::string name;
     Profile profile;
@@ -256,7 +261,7 @@ ProfileDialog :: run_until_valid (ProfileDialog& pd)
       gtk_widget_destroy (d);
       gtk_widget_grab_focus (pd._address_entry);
     }
-    else break;
+    else return true;
   }
 }
 
@@ -376,15 +381,16 @@ ProfilesDialog :: edit_profile ()
     if (_profiles.get_profile (old_name, profile))
     {
       ProfileDialog d (_data, old_name, profile, GTK_WINDOW(_root));
-      ProfileDialog :: run_until_valid (d);
-
-      std::string new_name;
-      d.get_profile (new_name, profile);
-      _profiles.delete_profile (old_name);
-      _profiles.add_profile (new_name, profile);
-      rebuild_store ();
-      refresh_buttons ();
-
+      const bool do_rebuild = ProfileDialog :: run_until_valid_or_cancel (d);
+      if (do_rebuild)
+      {
+        std::string new_name;
+        d.get_profile (new_name, profile);
+        _profiles.delete_profile (old_name);
+        _profiles.add_profile (new_name, profile);
+        rebuild_store ();
+        refresh_buttons ();
+      }
       gtk_widget_destroy (d.root());
     }
   }
@@ -397,13 +403,16 @@ ProfilesDialog :: create_new_profile ()
   profile.username = g_get_real_name ();
   profile.attribution = _("On %d, %n wrote:");
   ProfileDialog d (_data, _("New Profile"), profile, GTK_WINDOW(_root));
-  ProfileDialog :: run_until_valid (d);
-  std::string name;
-  d.get_profile (name, profile);
-  _profiles.add_profile (name, profile);
+  const bool do_rebuild = ProfileDialog :: run_until_valid_or_cancel (d);
+  if (do_rebuild)
+  {
+    std::string name;
+    d.get_profile (name, profile);
+    _profiles.add_profile (name, profile);
+    rebuild_store ();
+    refresh_buttons ();
+  }
   gtk_widget_destroy (d.root());
-  rebuild_store ();
-  refresh_buttons ();
 }
 
 void
