@@ -89,6 +89,7 @@ SaveDialog :: response_cb (GtkDialog * dialog,
   {
     SaveDialog * self (static_cast<SaveDialog*>(user_data));
 
+    // get the save path
     std::string path;
     bool use_custom_path (true);
     GtkWidget * w (self->_save_group_path_radio);
@@ -103,6 +104,13 @@ SaveDialog :: response_cb (GtkDialog * dialog,
     }
     path = expand_download_dir (path.c_str(), self->_group.to_view());
 
+    // get the save mode
+    int save_mode (0);
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->_save_text_check)))
+      save_mode |= TaskArticle::RAW;
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->_save_attachments_check)))
+      save_mode |= TaskArticle::DECODE;
+
     // make the tasks... 
     std::vector<Task*> tasks;
     foreach_const (std::vector<Article>, self->_articles, it)
@@ -112,6 +120,7 @@ SaveDialog :: response_cb (GtkDialog * dialog,
                                         self->_cache,
                                         self->_read,
                                         0,
+                                        TaskArticle::SaveMode(save_mode),
                                         path));
 
     // get the queue mode...
@@ -217,7 +226,7 @@ SaveDialog :: SaveDialog (Prefs                       & prefs,
     }
 
   HIG::workarea_add_section_divider (t, &row);
-  HIG::workarea_add_section_title (t, &row, _("Group Management"));
+  HIG::workarea_add_section_title (t, &row, _("Priority"));
     HIG::workarea_add_section_spacer (t, row, 3);
 
     // sort by age
