@@ -37,7 +37,8 @@ namespace pan
   class NNTP_Pool:
     public NNTP::Source,
     private NNTP::Listener,
-    private ServerInfo::Listener
+    private ServerInfo::Listener,
+    private Socket::Creator::Listener
   {
     public:
 
@@ -77,6 +78,9 @@ namespace pan
     private: // ServerInfo::Listener
       virtual void on_server_limits_changed (const Quark& server, int max_connections);
 
+    private: // Socket::Creator::Listener
+      virtual void on_socket_created (const StringView& host, int port, bool ok, Socket*);
+
     private:
 
       void fire_pool_has_nntp_available () {
