@@ -126,7 +126,7 @@ namespace
       }
       mc.a.message_id = mc.a.parts[0].message_id;
       const StringView p (mc.path.empty() ? mc.fallback_path : StringView(mc.path));
-      mc.tasks.push_back (new TaskArticle (mc.ranks, mc.gs, mc.a, mc.cache, mc.read, 0, p));
+      mc.tasks.push_back (new TaskArticle (mc.ranks, mc.gs, mc.a, mc.cache, mc.read, 0, TaskArticle::DECODE, p));
     }
   }
 
