@@ -251,7 +251,9 @@ Queue :: find_first_task_needing_server (const Quark& server)
     const Task::State& state ((*it)->get_state ());
     if  ((state._health != FAIL)
       && (state._work == Task::NEED_NNTP)
-      && (state._servers.count(server)))
+      && (state._servers.count(server))
+      && (!_stopped.count (*it))
+      && (!_removing.count (*it)))
     {
       retval = *it;
       break;
