@@ -23,6 +23,7 @@
 #include <map>
 #include <set>
 #include <vector>
+#include <pan/general/map-vector.h>
 #include <pan/general/quark.h>
 #include <pan/tasks/nntp-pool.h>
 #include <pan/tasks/socket.h>
@@ -35,8 +36,6 @@ namespace pan
   class NNTP;
   class ServerInfo;
 
-  typedef AdaptableSet<Task*, TaskWeakOrdering> TaskSet;
-
   /**
    * A Queue helper that saves tasks to disk and restores them from disk.
    *
@@ -56,12 +55,12 @@ namespace pan
    * @ingroup tasks
    */
   class Queue:
-    protected TaskSet,
     public NNTP::Source,
-    private NNTP_Pool::Listener
+    private NNTP_Pool::Listener,
+    private AdaptableSet<Task*, TaskWeakOrdering>::Listener
   {
     public:
-      Queue (const ServerInfo&, TaskArchive&, Socket::Creator*, bool online);
+      Queue (ServerInfo&, TaskArchive&, Socket::Creator*, bool online);
       virtual ~Queue ();
 
       typedef std::vector<Task*> tasks_t;
@@ -102,14 +101,25 @@ namespace pan
        * An ordered collection of tasks and their corresponding TaskState s.
        */ 
       struct task_states_t {
-        tasks_t tasks;
-        typedef std::map<Task*,TaskState> states_t;
-        states_t states;
-        TaskState get_state (Task* task) const {
-          states_t::const_iterator it = states.find (task);
-          return it == states.end() ? STOPPED : it->second;
-        }
+        friend class Queue;
+        private:
+          typedef sorted_vector<Task*,true> sorted_tasks_t;
+          sorted_tasks_t _queued;
+          sorted_tasks_t _stopped;
+          sorted_tasks_t _running;
+          sorted_tasks_t _removing;
+        public:
+          tasks_t tasks;
+          TaskState get_state (Task* task) const {
+            if (_removing.count(task)) return REMOVING;
+            if (_running.count(task)) return RUNNING;
+            if (_stopped.count(task)) return STOPPED;
+            if (_queued.count(task)) return QUEUED;
+            return STOPPED;
+          }
       };
+      Task* operator[](size_t i) { return _tasks[i]; }
+      const Task* operator[](size_t i) const { return _tasks[i]; }
       void get_all_task_states (task_states_t& setme);
       void get_task_counts (int& active, int& total);
 
@@ -139,10 +149,11 @@ namespace pan
 
     protected:
       void process_task (Task *);
-      const ServerInfo& _server_info;
+      void give_task_a_connection (Task*, NNTP*);
+      ServerInfo& _server_info;
       bool _is_online;
       Task* find_first_task_needing_server (const Quark& server);
-      bool find_best_server (const quarks_t& quarks, Quark& setme);
+      bool find_best_server (const Task::State::unique_servers_t& servers, Quark& setme);
       bool task_is_active (const Task*) const;
 
       typedef std::map<NNTP*,Task*> nntp_to_task_t;
@@ -169,12 +180,25 @@ namespace pan
       void fire_queue_error (const std::string& message);
 
     private:
-      typedef std::map<Quark,NNTP_Pool*> pools_t;
+      typedef Loki::AssocVector<Quark,NNTP_Pool*> pools_t;
       pools_t _pools;
       NNTP_Pool& get_pool (const Quark& server);
+      void request_connections (const quarks_t& servers);
+
+    private:
+      bool _needs_saving;
+      time_t _last_time_saved;
+      quarks_t _mids;
 
     private:
       TaskArchive& _archive;
+
+    private:
+      typedef AdaptableSet<Task*, TaskWeakOrdering> TaskSet;
+      TaskSet _tasks;
+      virtual void on_set_items_added  (TaskSet&, TaskSet::items_t&, int index);
+      virtual void on_set_item_removed (TaskSet&, Task*&, int index);
+      virtual void on_set_item_moved   (TaskSet&, Task*&, int index, int old_index);
   };
 }
 
