@@ -38,7 +38,7 @@ extern "C" {
 
 #ifdef G_OS_WIN32
   // this #define is necessary for mingw
-  #define _WIN32_WINNT 0x0500
+  #define _WIN32_WINNT 0x0501
   #include <ws2tcpip.h>
   #undef gai_strerror
   #define gai_strerror(i) gai_strerror_does_not_link (i)
@@ -78,6 +78,7 @@ extern "C" {
   #include <netinet/in.h>
   #include <netdb.h>
   #include <arpa/inet.h>
+  #define closesocket(fd) close(fd)
 #endif
 
 #include <pan/general/debug.h>
@@ -109,7 +110,6 @@ namespace
   {
     int err;
     int sockfd;
-    Log::add_info_va (_("Making a new connection to %*.*s"), host.len, host.len, host.str);
     ensure_module_inited ();
 
 #ifndef G_OS_WIN32
@@ -165,82 +165,65 @@ namespace
     if (!err)
       OutputDebugString ("Connection seems to be opened");
     else {
+      closesocket (sockfd);
       PrintLastError(err);
       return NULL;
     }
 
 #else	
 
-    struct addrinfo hints, *ans;
-    GList * l=NULL, *list=NULL, *ipv4_list=NULL, *ipv6_list=NULL;
-
+    struct addrinfo hints;
     memset (&hints, 0, sizeof(struct addrinfo));
     hints.ai_flags = 0;
     hints.ai_family = 0;
     hints.ai_socktype = SOCK_STREAM;
+    struct addrinfo * ans;
     err = ::getaddrinfo (hostbuf, portbuf, &hints, &ans);
     if (err != 0) {
       g_message ("Lookup error: %s", gai_strerror(err));
       return NULL;
     }
 
-    // build lists of ipv6 and ipv4 addrinfos 
+    // try to open a socket on any ipv4 or ipv6 addresses we found
+    sockfd = -1;
+    for (struct addrinfo * walk(ans); walk && sockfd<0; walk=walk->ai_next)
     {
-      struct addrinfo *answalk;
-      for (answalk=ans; answalk!=NULL; answalk=answalk->ai_next)
-      {
-        if (answalk->ai_family == PF_INET)
-          ipv4_list = g_list_append (ipv4_list, answalk);
-        else if (answalk->ai_family == PF_INET6)
-          ipv6_list = g_list_append (ipv6_list, answalk);
+      // only use ipv4 or ipv6 addresses
+      if ((walk->ai_family!=PF_INET) && (walk->ai_family!=PF_INET6))
+        continue;
+
+      // try to create a socket...
+      sockfd = ::socket (walk->ai_family, walk->ai_socktype, walk->ai_protocol);
+      if (sockfd < 0)
+        continue;
+
+      // and make a connection
+      if (::connect (sockfd, ans->ai_addr, ans->ai_addrlen) < 0) {
+        g_message ("Connect failed: %s", g_strerror(errno));
+        closesocket (sockfd);
+        sockfd = -1;
       }
     }
-
-    // merge the lists, putting ipv6 addrinfos before the ipv4
-    list = NULL;
-    list = g_list_concat (list, ipv6_list);
-    list = g_list_concat (list, ipv4_list);
-    ipv4_list = ipv6_list = NULL;
-
-    // walk through the merged list until we get a good socket
-    sockfd = -1;
-    for (l=list; l!=NULL; l=l->next)
-    {
-      struct addrinfo * ainfo = (struct addrinfo*) l->data;
-      sockfd = ::socket (ainfo->ai_family,
-                         ainfo->ai_socktype,
-                         ainfo->ai_protocol);
-      if (sockfd >= 0)
-        break;
-    }
-
-    g_list_free (list);
-    if (sockfd < 0) {
-      g_message ("couldn't create a socket.");
-      ::freeaddrinfo (ans);
-      return NULL;
-    }
-
-    // try to connect to the host
-    if (connect (sockfd, ans->ai_addr, ans->ai_addrlen) < 0) {
-      g_message ("Connect failed: %s", g_strerror(errno));
-      ::freeaddrinfo (ans);
-      return NULL;
-    }
-#endif
-
-    GIOChannel * channel = g_io_channel_unix_new (sockfd);
-#ifndef G_OS_WIN32
-    g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, NULL);
-#endif
-    if (g_io_channel_get_encoding (channel) != NULL)
-      g_io_channel_set_encoding (channel, NULL, NULL);
-    g_io_channel_set_buffered (channel, TRUE);
-    g_io_channel_set_line_term (channel, "\r\n", 2);
-
-#if !defined(G_OS_WIN32) || _WIN32_WINNT > 0x0500
     ::freeaddrinfo (ans);
+
 #endif
+
+    GIOChannel * channel (0);
+    if (sockfd < 0)
+      g_message ("couldn't create a socket.");
+    else {
+#ifndef G_OS_WIN32
+      channel = g_io_channel_unix_new (sockfd);
+      g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, NULL);
+#else
+      channel = g_io_channel_win32_new_socket (sockfd);
+#endif
+      if (g_io_channel_get_encoding (channel) != NULL)
+        g_io_channel_set_encoding (channel, NULL, NULL);
+      g_io_channel_set_buffered (channel, true);
+      g_io_channel_set_line_term (channel, "\r\n", 2);
+    }
+
     return channel;
   }
 }
@@ -261,15 +244,22 @@ GIOChannelSocket :: GIOChannelSocket ( ):
    debug ("GIOChannelSocket ctor " << (void*)this);
 }
 
+namespace
+{
+  void remove_source (guint& tag) {
+    if (tag) {
+      g_source_remove (tag);
+      tag = 0;
+    }
+  }
+}
+
 GIOChannelSocket :: ~GIOChannelSocket ()
 {
 //std::cerr << LINE_ID << " destroying socket " << this << std::endl;
 
-  if (_tag_watch)
-  {
-    g_source_remove (_tag_watch);
-    _tag_watch = 0;
-  }
+  remove_source (_tag_watch);
+  remove_source (_tag_timeout);
 
   if (_channel)
   {
@@ -285,18 +275,11 @@ GIOChannelSocket :: ~GIOChannelSocket ()
   _in_buf = 0;
 }
 
-void
-GIOChannelSocket :: open (const StringView& address, int port, Listener * l)
+bool
+GIOChannelSocket :: open (const StringView& address, int port)
 {
-  _listener = l;
   _channel = create_channel (address, port);
-
-  if (_listener) {
-    if (_channel)
-      _listener->onSocketOpen (this);
-    else
-      _listener->onSocketError (this);
-  }
+  return _channel != 0;
 }
 
 void
@@ -335,7 +318,7 @@ GIOChannelSocket :: do_read ()
          // strip off \r\n
          if (g->len>=2 && g->str[g->len-2]=='\r' && g->str[g->len-1]=='\n')
             g_string_truncate (g, g->len-2);
-         more = _listener->onSocketResponse (this, StringView (g->str, g->len));
+         more = _listener->on_socket_response (this, StringView (g->str, g->len));
       }
       else if (status == G_IO_STATUS_AGAIN)
       {
@@ -348,7 +331,7 @@ GIOChannelSocket :: do_read ()
          Log::add_err_va (_("Error reading from socket: %s"), msg);
          if (err != NULL)
             g_clear_error (&err);
-         _listener->onSocketError (this);
+         _listener->on_socket_error (this);
          more = false;
       }
    }
@@ -356,7 +339,7 @@ GIOChannelSocket :: do_read ()
    // if more is false, 'this' may have been deleted by the listeners...
    // so don't touch _abort_flag or _listener unless we're clean here.
    if (more && _abort_flag)
-      _listener->onSocketAbort (this);
+      _listener->on_socket_abort (this);
 
    return false;
 }
@@ -370,7 +353,7 @@ GIOChannelSocket :: do_write ()
    if (_abort_flag)
    {
       finished = true;
-      _listener->onSocketAbort (this);
+      _listener->on_socket_abort (this);
    }
    else
    {
@@ -393,7 +376,7 @@ GIOChannelSocket :: do_write ()
       if (err) {
          Log::add_err (err->message);
          g_clear_error (&err);
-         _listener->onSocketError (this);
+         _listener->on_socket_error (this);
          return false;
       }
 
@@ -413,6 +396,15 @@ GIOChannelSocket :: do_write ()
    return !finished;
 }
 
+gboolean
+GIOChannelSocket :: timeout_func (gpointer sock_gp)
+{
+  GIOChannelSocket * self (static_cast<GIOChannelSocket*>(sock_gp));
+  debug ("error: channel " << self->_channel << " not responding.");
+  gio_func (self->_channel, G_IO_ERR, sock_gp);
+  return false;
+}
+
 gboolean
 GIOChannelSocket :: gio_func (GIOChannel * channel, GIOCondition cond, gpointer sock_gp)
 {
@@ -423,14 +415,14 @@ GIOChannelSocket :: gio_func (GIOChannel * channel, GIOCondition cond, gpointer
    if (self->_abort_flag)
    {
       self->set_watch_mode (IGNORE_NOW);
-      self->_listener->onSocketAbort (self);
+      self->_listener->on_socket_abort (self);
       gimmie_more = false;
    }
    else if (!(cond & (G_IO_IN | G_IO_OUT)))
    {
       debug ("socket " << self << " got err or hup; ignoring");
       self->set_watch_mode (IGNORE_NOW);
-      self->_listener->onSocketError (self);
+      self->_listener->on_socket_error (self);
       gimmie_more = false;
    }
    else if (cond & G_IO_IN)
@@ -462,44 +454,94 @@ namespace
   const double SECONDS_TO_SLEEP_ON_EAGAIN (0.2);
 
   const unsigned int SLEEP_MSEC ((unsigned int)(SECONDS_TO_SLEEP_ON_EAGAIN * 1000.0));
+
+  const unsigned int TIMEOUT_SECS (30);
 }
 
 void
 GIOChannelSocket :: set_watch_mode (WatchMode mode)
 {
-   debug ("socket " << this << " calling set_watch_mode " << mode << "; _channel is " << _channel);
+  debug ("socket " << this << " calling set_watch_mode " << mode << "; _channel is " << _channel);
+  remove_source (_tag_watch);
+  remove_source (_tag_timeout);
 
-   if (_tag_watch)
-   {
-      g_source_remove (_tag_watch);
-      _tag_watch = 0;
-   }
+  guint cond;
+  switch (mode)
+  {
+    case IGNORE_NOW:
+      // don't add any watches
+      debug("channel " << _channel << " setting mode **IGNORE**");
+      break;
 
-   guint cond;
-   switch (mode)
-   {
-      case IGNORE_NOW:
-         // don't add any watches
-         debug("channel " << _channel << " setting mode **IGNORE**");
-         break;
+    case READ_NOW:
+      debug("channel " << _channel << " setting mode read");
+      cond = (int)G_IO_IN | (int)G_IO_ERR | (int)G_IO_HUP | (int)G_IO_NVAL;
+      _tag_watch = g_io_add_watch (_channel, (GIOCondition)cond, gio_func, this);
+      _tag_timeout = g_timeout_add (TIMEOUT_SECS*1000, timeout_func, this);
+      break;
 
-      case READ_NOW:
-         debug("channel " << _channel << " setting mode read");
-         cond = (int)G_IO_IN | (int)G_IO_ERR | (int)G_IO_HUP | (int)G_IO_NVAL;
-         _tag_watch = g_io_add_watch (_channel, (GIOCondition)cond, gio_func, this);
-         break;
+    case READ_LATER:
+      debug("channel " << _channel << " setting mode read later");
+      _tag_watch = g_timeout_add (SLEEP_MSEC, read_later, this);
+      break;
 
-      case READ_LATER:
-         debug("channel " << _channel << " setting mode read later");
-         _tag_watch = g_timeout_add (SLEEP_MSEC, read_later, this);
-         break;
+    case WRITE_NOW:
+      debug("channel " << _channel << " setting mode write");
+      cond = (int)G_IO_OUT | (int)G_IO_ERR | (int)G_IO_HUP | (int)G_IO_NVAL;
+      _tag_watch = g_io_add_watch (_channel, (GIOCondition)cond, gio_func, this);
+      _tag_timeout = g_timeout_add (TIMEOUT_SECS*1000, timeout_func, this);
+      break;
+  }
 
-      case WRITE_NOW:
-         debug("channel " << _channel << " setting mode write");
-         cond = (int)G_IO_OUT | (int)G_IO_ERR | (int)G_IO_HUP | (int)G_IO_NVAL;
-         _tag_watch = g_io_add_watch (_channel, (GIOCondition)cond, gio_func, this);
-         break;
-   }
-
-   debug ("set_watch_mode " << mode << ": _tag_watch is now " << _tag_watch);
+  debug ("set_watch_mode " << mode << ": _tag_watch is now " << _tag_watch);
+}
+
+/***
+****  GIOChannel::SocketCreator -- create a socket in a worker thread
+***/
+
+namespace
+{
+  struct ThreadInfo
+  {
+    std::string host;
+    int port;
+    Socket::Creator::Listener * listener;
+
+    bool ok;
+    Socket * socket;
+
+    ThreadInfo (const StringView& h, int p, Socket::Creator::Listener *l):
+      host(h), port(p), listener(l), ok(false), socket(0) {}
+  };
+
+  gboolean socket_created_idle (gpointer info_gpointer)
+  {
+    ThreadInfo * info (static_cast<ThreadInfo*>(info_gpointer));
+    info->listener->on_socket_created (info->host, info->port, info->ok, info->socket);
+    delete info;
+    return false;
+  }
+
+  gpointer create_socket_thread_func (gpointer info_gpointer)
+  {
+    //std::cerr << LINE_ID << " creating a socket in a separate thread..." << std::endl;
+    ThreadInfo * info (static_cast<ThreadInfo*>(info_gpointer));
+    info->socket = new GIOChannelSocket ();
+    info->ok = info->socket->open (info->host, info->port);
+    g_idle_add (socket_created_idle, info); // pass the results back to the main thread...
+    return 0;
+  }
+}
+  
+void
+GIOChannelSocket :: Creator :: create_socket (const StringView & host,
+                                              int                port,
+                                              Listener         * listener)
+{
+  // farm this out to a worker thread so that the main thread
+  // doesn't block while we open the socket.
+  g_thread_create (create_socket_thread_func,
+                   new ThreadInfo (host, port, listener),
+                   true, 0);
 }
