@@ -42,14 +42,13 @@ namespace pan
       /** Interface class for objects that listen to a Socket's events */
       struct Listener {
         virtual ~Listener () {}
-        virtual void onSocketOpen (Socket*) = 0;
-        virtual bool onSocketResponse (Socket*, const StringView& line) = 0;
-        virtual void onSocketError (Socket*) = 0;
-        virtual void onSocketAbort (Socket*) = 0;
+        virtual bool on_socket_response (Socket*, const StringView& line) = 0;
+        virtual void on_socket_error (Socket*) = 0;
+        virtual void on_socket_abort (Socket*) = 0;
       };
 
     public:
-      virtual void open (const StringView& address, int port, Listener *) = 0;
+      virtual bool open (const StringView& address, int port) = 0;
       virtual void write_command (const StringView& chars, Listener *) = 0;
 
     public:
@@ -76,9 +75,15 @@ namespace pan
        * 
        * @ingroup tasks
        */
-      struct Creator {
+      struct Creator
+      {
+        struct Listener {
+          virtual ~Listener () {}
+          virtual void on_socket_created (const StringView& host, int port, bool ok, Socket*) = 0;
+        };
+
         virtual ~Creator () { }
-        virtual Socket* create_socket (const StringView& host, int port) = 0;
+        virtual void create_socket (const StringView& host, int port, Listener*) = 0;
       };
   };
 }
