@@ -64,6 +64,7 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
                             ArticleCache              & cache,
                             ArticleRead               & read,
                             Task::Listener            * listener,
+                            SaveMode                    save_mode,
                             const StringView          & save_path):
   Task (save_path.empty() ? "BODIES" : "SAVE", get_description (article, !save_path.empty())),
   _save_path (save_path.str, save_path.len),
@@ -73,7 +74,7 @@ TaskArticle :: TaskArticle (const ServerRank          & server_rank,
   _article (article),
   _time_posted (article.time_posted),
   _finished_proc_has_run (false),
-  _save_when_finished (!save_path.empty())
+  _save_mode (save_mode)
 {
   cache.reserve (article.parts);
 
@@ -281,9 +282,35 @@ namespace
 void
 TaskArticle :: on_finished ()
 {
-  if (_save_when_finished)
+  if (_save_mode & RAW)
   {
-    const ArticleCache :: strings_t filenames = _cache.get_filenames (_article.parts);
+    const ArticleCache :: strings_t filenames (_cache.get_filenames (_article.parts));
+    foreach_const (ArticleCache::strings_t, filenames, it)
+    {
+      gchar * contents (0);
+      gsize length (0);
+      if (g_file_get_contents (it->c_str(), &contents, &length, NULL) && length>0)
+      {
+        file :: ensure_dir_exists (_save_path);
+        gchar * basename (g_path_get_basename (it->c_str()));
+        gchar * filename (g_build_filename (_save_path.c_str(), basename, NULL));
+        FILE * fp = fopen (filename, "w+");
+        if (!fp)
+          Log::add_err_va (_("Couldn't save file \"%s\": %s"), filename, file::pan_strerror(errno));
+        else {
+          fwrite (contents, 1, (size_t)length, fp);
+          fclose (fp);
+        }
+        g_free (filename);
+        g_free (basename);
+      }
+      g_free (contents);
+    }
+  }
+
+  if (_save_mode & DECODE)
+  {
+    const ArticleCache :: strings_t filenames (_cache.get_filenames (_article.parts));
 
     // decode
     int res;
@@ -297,7 +324,6 @@ TaskArticle :: on_finished ()
 
       int i (0);
       foreach_const (ArticleCache::strings_t, filenames, it) {
-//std::cerr << LINE_ID << " loading " << *it << " as part " << (i+1) << std::endl;
         if ((res = UULoadFileWithPartNo (const_cast<char*>(it->c_str()), 0, 0, ++i)) != UURET_OK)
           Log::add_err_va (_("Couldn't load \"%s\": %s"), it->c_str(),
             (res==UURET_IOERR) ?  file::pan_strerror (UUGetOption (UUOPT_ERRNO, NULL, NULL, 0)) : UUstrerror(res));
