@@ -37,12 +37,16 @@ namespace pan
   class TaskArticle: public Task, private NNTP::Listener
   {
     public: // life cycle
+
+      enum SaveMode { NONE=0, DECODE=(1<<0), RAW=(1<<1) };
+
       TaskArticle (const ServerRank   & server_rank,
                    const GroupServer  & group_server,
                    const Article      & article,
                    ArticleCache       & cache,
                    ArticleRead        & read,
                    Task::Listener     * l=0,
+                   SaveMode             save_mode = NONE,
                    const StringView   & save_path = StringView());
       virtual ~TaskArticle ();
       time_t get_time_posted () const { return _time_posted; }
@@ -71,7 +75,7 @@ namespace pan
 
     private: // implementation
       bool _finished_proc_has_run;
-      const bool _save_when_finished;
+      const SaveMode _save_mode;
       typedef std::map<Quark,int> stats_t;
       stats_t _stats;
 
